=== modified file 'HACKING'
--- HACKING	2006-08-01 18:27:42 +0000
+++ HACKING	2006-09-13 02:09:37 +0000
@@ -209,6 +209,65 @@
 > factory, then yes, foo_factory is what I would use.
 
 
+Lazy Imports
+------------
+
+To make startup time faster, we use the ``bzrlib.lazy_import`` module to
+delay importing modules until they are actually used. ``lazy_import`` uses
+the same syntax as regular python imports. So to import a few modules in a
+lazy fashion do::
+
+  from bzrlib.lazy_import import lazy_import
+  lazy_import(globals(), """
+  import os
+  import subprocess
+  import sys
+  import time
+
+  from bzrlib import (
+     errors,
+     transport,
+     foo as bar,
+     )
+  import bzrlib.transport
+  import bzrlib.xml5
+  """)
+
+At this point, all of these exist as a ``ImportReplacer`` object, ready to
+be imported once a member is accessed.
+
+
+Modules versus Members
+~~~~~~~~~~~~~~~~~~~~~~
+
+While it is possible for ``lazy_import()`` to import members of a module
+wehn using the ``from module import member`` syntax, it is recommended to
+only use that syntax to load sub modules ``from module import submodule``.
+This is because variables and classes can frequently be used without
+needing a sub-member for example::
+
+  lazy_import(globals(), """
+  from module import MyClass
+  """)
+
+  def test(x):
+      return isinstance(x, MyClass)
+
+This will incorrectly fail, because ``MyClass`` is a ``ImportReplacer``
+object, rather than the real class.
+
+
+Passing to other variables
+~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+It also is incorrect to assign ``ImportReplacer`` objects to other variables.
+Because the replacer only knows about the original name, it is unable to
+replace other variables. The ``ImportReplacer`` class will raise an
+``IllegalUseOfScopeReplacer`` exception if it can figure out that this
+happened. But it requires accessing a member more than once from the new
+variable, so some bugs are not detected right away.
+
+
 Writing output
 ==============
 

=== modified file 'NEWS'
--- NEWS	2006-09-08 00:19:48 +0000
+++ NEWS	2006-09-13 01:57:42 +0000
@@ -16,8 +16,20 @@
 
     * Commit performs one less XML parse. (Robert Collins)
 
+    * ``bzr checkout --lightweight`` now operates on readonly branches as well
+      as readwrite branches. This fixes bug #39542 for lightweight checkouts 
+      but not for heavyweight ones (the default). (Robert Collins)
+
+    * ``bzrlib.lazy_import.lazy_import`` function to create on-demand 
+      objects.  This allows all imports to stay at the global scope, but
+      modules will not actually be imported if they are not used.
+      (John Arbash Meinel)
+
   BUG FIXES:
 
+    * Don't access e.code for generic URLErrors, only HTTPErrors have .code.
+      (Vincent Ladeuil, #59835)
+
     * Handle boundary="" lines properly to allow access through a Squid proxy.
       (John Arbash Meinel, #57723)
 
@@ -76,7 +88,29 @@
     * WorkingTree.pending_merges is deprecated.  Please use the get_parent_ids
       (introduced in 0.10) method instead. (Robert Collins)
 
-bzr 0.10.0RC1  2006-08-28
+    * WorkingTree has a new lock_tree_write method which locks the branch for
+      read rather than write. This is appropriate for actions which only need
+      the branch data for reference rather than mutation. A new decorator
+      needs_tree_write_lock is provided in the workingtree module. Like the
+      needs_read_lock and needs_write_lock decorators this allows static 
+      declaration of the locking requirements of a function to ensure that
+      a lock is taken out for casual scripts. (Robert Collins)
+
+    * All WorkingTree methods which write to the tree, but not to the branch
+      have been converted to use ``needs_tree_write_lock`` rather than 
+      ``needs_write_lock``. Also converted is the revert, conflicts and tree
+      transform modules. This provides a modest performance improvement on 
+      metadir style trees, due to the reduce lock-acquisition, and a more
+      significant performance improvement on lightweight checkouts from 
+      remote branches, where trivial operations used to pay a significant 
+      penalty. It also provides the basis for allowing readonly checkouts.
+      (Robert Collins)
+
+    * Special case importing the standard library 'copy' module. This shaves
+      off 40ms of startup time, while retaining compatibility. See:
+      ``bzrlib/inspect_for_copy.py`` for more details. (John Arbash Meinel)
+
+bzr 0.10  2006-08-29
   
   IMPROVEMENTS:
     * 'merge' now takes --uncommitted, to apply uncommitted changes from a
@@ -135,6 +169,9 @@
       in handling of revision properties. (John Arbash Meinel, Holger Krekel,
       #54723)
 
+    * The bzr selftest was failing on installed versions due to a bug in a new
+      test helper. (John Arbash Meinel, Robert Collins, #58057)
+
   INTERNALS:
 
     * ``bzrlib.cache_utf8`` contains ``encode()`` and ``decode()`` functions

=== modified file 'README'
--- README	2006-03-16 20:20:32 +0000
+++ README	2006-09-12 21:56:31 +0000
@@ -1,8 +1,8 @@
-*******************************
-Release notes for Bazaar-NG 0.8
-*******************************
+*****************************
+Release notes for Bazaar 0.10
+*****************************
 
-bazaar-ng is a distributed/decentralized version control system.
+Bazaar is a distributed/decentralized version control system.
 
 Features include:
 

=== modified file 'bzr'
--- bzr	2006-08-28 01:09:52 +0000
+++ bzr	2006-09-13 01:52:31 +0000
@@ -61,8 +61,6 @@
 
 try:
     import bzrlib
-    import bzrlib.commands
-    import bzrlib.trace
 except ImportError, e:
     sys.stderr.write("bzr: ERROR: "
                      "Couldn't import bzrlib and dependencies.\n"
@@ -70,6 +68,12 @@
                      "\n")
     raise
 
+import bzrlib.inspect_for_copy
+bzrlib.inspect_for_copy.import_copy_with_hacked_inspect()
+
+import bzrlib.commands
+import bzrlib.trace
+
 if bzrlib.version_info[:3] != (0, 11, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-09-13 10:36:03 +0000
+++ bzrlib/branch.py	2006-09-14 04:10:40 +0000
@@ -581,6 +581,18 @@
             mainline_parent_id = revision_id
         return BranchCheckResult(self)
 
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repostory uses weaves.
+        """
+        if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
+            from bzrlib import repository
+            format = bzrdir.BzrDirMetaFormat1()
+            format.repository_format = repository.RepositoryFormat7()
+        else:
+            format = self.repository.bzrdir.cloning_metadir()
+        return format
+
     def create_checkout(self, to_location, revision_id=None, 
                         lightweight=False):
         """Create a checkout of a branch.
@@ -591,17 +603,18 @@
         produce a bound branch (heavyweight checkout)
         :return: The tree of the created checkout
         """
+        t = transport.get_transport(to_location)
+        try:
+            t.mkdir('.')
+        except errors.FileExists:
+            pass
         if lightweight:
-            t = transport.get_transport(to_location)
-            try:
-                t.mkdir('.')
-            except errors.FileExists:
-                pass
             checkout = bzrdir.BzrDirMetaFormat1().initialize_on_transport(t)
             BranchReferenceFormat().initialize(checkout, self)
         else:
+            format = self._get_checkout_format()
             checkout_branch = bzrdir.BzrDir.create_branch_convenience(
-                to_location, force_new_tree=False)
+                to_location, force_new_tree=False, format=format)
             checkout = checkout_branch.bzrdir
             checkout_branch.bind(self)
             if revision_id is not None:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-09-08 18:46:29 +0000
+++ bzrlib/builtins.py	2006-09-12 01:40:25 +0000
@@ -107,6 +107,10 @@
         format = bzrdir.BzrDirMetaFormat1()
         format.repository_format = repository.RepositoryFormatKnit1()
         return format
+    if typestring == "experimental-knit2":
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        return format
     msg = "Unknown bzr format %s. Current formats are: default, knit,\n" \
           "metaweave and weave" % typestring
     raise BzrCommandError(msg)
@@ -760,7 +764,7 @@
         # if the source and to_location are the same, 
         # and there is no working tree,
         # then reconstitute a branch
-        if (osutils.abspath(to_location) == 
+        if (osutils.abspath(to_location) ==
             osutils.abspath(branch_location)):
             try:
                 source.bzrdir.open_workingtree()
@@ -1346,7 +1350,12 @@
         else:
             # local dir only
             # FIXME ? log the current subdir only RBC 20060203 
-            dir, relpath = bzrdir.BzrDir.open_containing('.')
+            if revision is not None \
+                    and len(revision) > 0 and revision[0].get_branch():
+                location = revision[0].get_branch()
+            else:
+                location = '.'
+            dir, relpath = bzrdir.BzrDir.open_containing(location)
             b = dir.open_branch()
 
         if revision is None:
@@ -1355,6 +1364,12 @@
         elif len(revision) == 1:
             rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
+            if revision[1].get_branch() != revision[0].get_branch():
+                # b is taken from revision[0].get_branch(), and
+                # show_log will use its revision_history. Having
+                # different branches will lead to weird behaviors.
+                raise BzrCommandError(
+                    "Log doesn't accept two revisions in different branches.")
             if revision[0].spec is None:
                 # missing begin-range means first revision
                 rev1 = 1
@@ -1648,6 +1663,8 @@
 
         if tree is None:
             b, relpath = Branch.open_containing(filename)
+        if revision is not None and revision[0].get_branch() is not None:
+            b = Branch.open(revision[0].get_branch())
         if revision is None:
             revision_id = b.last_revision()
         else:
@@ -2133,7 +2150,9 @@
                 else:
                     return 1
 
-        branch = self._get_remembered_parent(tree, branch, 'Merging from')
+        if revision is None \
+                or len(revision) < 1 or revision[0].needs_branch():
+            branch = self._get_remembered_parent(tree, branch, 'Merging from')
 
         if revision is None or len(revision) < 1:
             if uncommitted:
@@ -2147,6 +2166,7 @@
             if uncommitted:
                 raise BzrCommandError('Cannot use --uncommitted and --revision'
                                       ' at the same time.')
+            branch = revision[0].get_branch() or branch
             if len(revision) == 1:
                 base = [None, None]
                 other_branch, path = Branch.open_containing(branch)
@@ -2156,11 +2176,16 @@
                 assert len(revision) == 2
                 if None in revision:
                     raise BzrCommandError(
-                        "Merge doesn't permit that revision specifier.")
-                other_branch, path = Branch.open_containing(branch)
+                        "Merge doesn't permit empty revision specifier.")
+                base_branch, path = Branch.open_containing(branch)
+                branch1 = revision[1].get_branch() or branch
+                other_branch, path1 = Branch.open_containing(branch1)
+                if revision[0].get_branch() is not None:
+                    # then path was obtained from it, and is None.
+                    path = path1
 
-                base = [branch, revision[0].in_history(other_branch).revno]
-                other = [branch, revision[1].in_history(other_branch).revno]
+                base = [branch, revision[0].in_history(base_branch).revno]
+                other = [branch1, revision[1].in_history(other_branch).revno]
 
         if tree.branch.get_parent() is None or remember:
             tree.branch.set_parent(other_branch.base)

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-09-13 10:36:03 +0000
+++ bzrlib/bzrdir.py	2006-09-14 04:10:40 +0000
@@ -93,6 +93,11 @@
         """Return true if this bzrdir is one whose format we can convert from."""
         return True
 
+    def check_conversion_target(self, target_format):
+        target_repo_format = target_format.repository_format
+        source_repo_format = self._format.repository_format
+        source_repo_format.check_conversion_target(target_repo_format)
+
     @staticmethod
     def _check_supported(format, allow_unsupported):
         """Check whether format is a supported format.
@@ -570,6 +575,28 @@
         except errors.NoWorkingTree:
             return False
 
+    def cloning_metadir(self, basis=None):
+        """Produce a metadir suitable for cloning with"""
+        def related_repository(bzrdir):
+            try:
+                branch = bzrdir.open_branch()
+                return branch.repository
+            except errors.NotBranchError:
+                source_branch = None
+                return bzrdir.open_repository()
+        result_format = self._format.__class__()
+        try:
+            try:
+                source_repository = related_repository(self)
+            except errors.NoRepositoryPresent:
+                if basis is None:
+                    raise
+                source_repository = related_repository(self)
+            result_format.repository_format = source_repository._format
+        except errors.NoRepositoryPresent:
+            pass
+        return result_format
+
     def sprout(self, url, revision_id=None, basis=None, force_new_repo=False):
         """Create a copy of this bzrdir prepared for use as a new line of
         development.
@@ -585,7 +612,8 @@
             itself to download less data.
         """
         self._make_tail(url)
-        result = self._format.initialize(url)
+        cloning_format = self.cloning_metadir(basis)
+        result = cloning_format.initialize(url)
         basis_repo, basis_branch, basis_tree = self._get_basis_components(basis)
         try:
             source_branch = self.open_branch()
@@ -1063,6 +1091,10 @@
         """
         return True
 
+    def same_model(self, target_format):
+        return (self.repository_format.rich_root_data == 
+            target_format.rich_root_data)
+
     @classmethod
     def known_formats(klass):
         """Return all the known formats.
@@ -1184,7 +1216,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat4
-        return RepositoryFormat4(self)
+        return RepositoryFormat4()
     repository_format = property(__return_repository_format)
 
 
@@ -1244,7 +1276,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat5
-        return RepositoryFormat5(self)
+        return RepositoryFormat5()
     repository_format = property(__return_repository_format)
 
 
@@ -1303,7 +1335,7 @@
     def __return_repository_format(self):
         """Circular import protection."""
         from bzrlib.repository import RepositoryFormat6
-        return RepositoryFormat6(self)
+        return RepositoryFormat6()
     repository_format = property(__return_repository_format)
 
 
@@ -1548,6 +1580,7 @@
         assert rev_id not in self.converted_revs
         old_inv_xml = self.branch.repository.inventory_store.get(rev_id).read()
         inv = serializer_v4.read_inventory_from_string(old_inv_xml)
+        inv.revision_id = rev_id
         rev = self.revisions[rev_id]
         if rev.inventory_sha1:
             assert rev.inventory_sha1 == sha_string(old_inv_xml), \

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-09-13 10:36:03 +0000
+++ bzrlib/commit.py	2006-09-14 04:10:40 +0000
@@ -297,10 +297,7 @@
             self._populate_new_inv()
             self._report_deletes()
 
-            if not (self.allow_pointless
-                    or len(self.parents) > 1
-                    or self.builder.new_inventory != self.basis_inv):
-                raise PointlessCommit()
+            self._check_pointless()
 
             self._emit_progress_update()
             # TODO: Now the new inventory is known, check for conflicts and
@@ -348,6 +345,21 @@
             self._cleanup()
         return self.rev_id
 
+    def _check_pointless(self):
+        if self.allow_pointless:
+            return
+        # A merge with no effect on files
+        if len(self.parents) > 1:
+            return
+        # work around the fact that a newly-initted tree does differ from its
+        # basis
+        if len(self.builder.new_inventory) != len(self.basis_inv):
+            return
+        if (len(self.builder.new_inventory) != 1 and
+            self.builder.new_inventory != self.basis_inv):
+            return
+        raise PointlessCommit()
+
     def _check_bound_branch(self):
         """Check to see if the local branch is bound.
 

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-09-06 12:00:01 +0000
+++ bzrlib/conflicts.py	2006-09-10 23:42:17 +0000
@@ -88,7 +88,7 @@
 
 
 def resolve(tree, paths=None, ignore_misses=False):
-    tree.lock_write()
+    tree.lock_tree_write()
     try:
         tree_conflicts = tree.conflicts()
         if paths is None:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-09-13 10:36:03 +0000
+++ bzrlib/errors.py	2006-09-14 04:10:40 +0000
@@ -990,6 +990,15 @@
         self.format = format
 
 
+class BadConversionTarget(BzrNewError):
+    """Cannot convert to format %(format)s.  %(problem)s"""
+
+    def __init__(self, problem, format):
+        BzrNewError.__init__(self)
+        self.problem = problem
+        self.format = format
+
+
 class NoDiff(BzrNewError):
     """Diff is not installed on this machine: %(msg)s"""
 
@@ -1148,6 +1157,17 @@
         BzrNewError.__init__(self)
 
 
+class BadInventoryFormat(BzrNewError):
+    """Root class for inventory serialization errors"""
+
+
+class UnexpectedInventoryFormat(BadInventoryFormat):
+    """The inventory was not in the expected format:\n %(msg)s"""
+
+    def __init__(self, msg):
+        BadInventoryFormat.__init__(self, msg=msg)
+
+
 class UnknownSSH(BzrNewError):
     """Unrecognised value for BZR_SSH environment variable: %(vendor)s"""
 
@@ -1162,3 +1182,39 @@
     def __init__(self, revision_id):
         BzrNewError.__init__(self)
         self.revision_id = revision_id
+
+
+class IllegalUseOfScopeReplacer(BzrNewError):
+    """ScopeReplacer object %(name)r was used incorrectly: %(msg)s%(extra)s"""
+
+    is_user_error = False
+
+    def __init__(self, name, msg, extra=None):
+        BzrNewError.__init__(self)
+        self.name = name
+        self.msg = msg
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class InvalidImportLine(BzrNewError):
+    """Not a valid import statement: %(msg)\n%(text)s"""
+
+    is_user_error = False
+
+    def __init__(self, text, msg):
+        BzrNewError.__init__(self)
+        self.text = text
+        self.msg = msg
+
+
+class ImportNameCollision(BzrNewError):
+    """Tried to import an object to the same name as an existing object. %(name)s"""
+
+    is_user_error = False
+
+    def __init__(self, name):
+        BzrNewError.__init__(self)
+        self.name = name

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-09-08 18:46:29 +0000
+++ bzrlib/fetch.py	2006-09-10 02:18:31 +0000
@@ -281,6 +281,107 @@
         to_rf.join(from_rf, version_ids=revs)
 
 
+class Inter1and2Helper(object):
+    """Helper for operations that convert data from model 1 and 2
+    
+    This is for use by fetchers and converters.
+    """
+
+    def __init__(self, source, target):
+        """Constructor.
+
+        :param source: The repository data comes from
+        :param target: The repository data goes to
+        """
+        self.source = source
+        self.target = target
+
+    def iter_rev_trees(self, revs):
+        """Iterate through RevisionTrees efficiently.
+
+        Additionally, the inventory's revision_id is set if unset.
+
+        Trees are retrieved in batches of 100, and then yielded in the order
+        they were requested.
+
+        :param revs: A list of revision ids
+        """
+        while revs:
+            for tree in self.source.revision_trees(revs[:100]):
+                if tree.inventory.revision_id is None:
+                    tree.inventory.revision_id = tree.get_revision_id()
+                yield tree
+            revs = revs[100:]
+
+    def generate_root_texts(self, revs):
+        """Generate VersionedFiles for all root ids.
+        
+        :param revs: the revisions to include
+        """
+        inventory_weave = self.source.get_inventory_weave()
+        parent_texts = {}
+        versionedfile = {}
+        to_store = self.target.weave_store
+        for tree in self.iter_rev_trees(revs):
+            revision_id = tree.inventory.root.revision
+            root_id = tree.inventory.root.file_id
+            parents = inventory_weave.get_parents(revision_id)
+            if root_id not in versionedfile:
+                versionedfile[root_id] = to_store.get_weave_or_empty(root_id, 
+                    self.target.get_transaction())
+            parent_texts[root_id] = versionedfile[root_id].add_lines(
+                revision_id, parents, [], parent_texts)
+
+    def regenerate_inventory(self, revs):
+        """Generate a new inventory versionedfile in target, convertin data.
+        
+        The inventory is retrieved from the source, (deserializing it), and
+        stored in the target (reserializing it in a different format).
+        :param revs: The revisions to include
+        """
+        inventory_weave = self.source.get_inventory_weave()
+        for tree in self.iter_rev_trees(revs):
+            parents = inventory_weave.get_parents(tree.get_revision_id())
+            self.target.add_inventory(tree.get_revision_id(), tree.inventory,
+                                      parents)
+
+
+class Model1toKnit2Fetcher(GenericRepoFetcher):
+    """Fetch from a Model1 repository into a Knit2 repository
+    """
+    def __init__(self, to_repository, from_repository, last_revision=None, 
+                 pb=None):
+        self.helper = Inter1and2Helper(from_repository, to_repository)
+        GenericRepoFetcher.__init__(self, to_repository, from_repository,
+                                    last_revision, pb)
+
+    def _fetch_weave_texts(self, revs):
+        GenericRepoFetcher._fetch_weave_texts(self, revs)
+        # Now generate a weave for the tree root
+        self.helper.generate_root_texts(revs)
+
+    def _fetch_inventory_weave(self, revs):
+        self.helper.regenerate_inventory(revs)
+ 
+
+class Knit1to2Fetcher(KnitRepoFetcher):
+    """Fetch from a Knit1 repository into a Knit2 repository"""
+
+    def __init__(self, to_repository, from_repository, last_revision=None, 
+                 pb=None):
+        self.helper = Inter1and2Helper(from_repository, to_repository)
+        KnitRepoFetcher.__init__(self, to_repository, from_repository,
+                                 last_revision, pb)
+
+    def _fetch_weave_texts(self, revs):
+        KnitRepoFetcher._fetch_weave_texts(self, revs)
+        # Now generate a weave for the tree root
+        self.helper.generate_root_texts(revs)
+
+    def _fetch_inventory_weave(self, revs):
+        self.helper.regenerate_inventory(revs)
+        
+
 class Fetcher(object):
     """Backwards compatibility glue for branch.fetch()."""
 

=== added file 'bzrlib/inspect_for_copy.py'
--- bzrlib/inspect_for_copy.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inspect_for_copy.py	2006-09-13 01:52:31 +0000
@@ -0,0 +1,74 @@
+# Copyright (C) 2005, 2006 Canonical Development Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""A version of inspect that includes what 'copy' needs.
+
+Importing the python standard module 'copy' is far more expensive than it
+needs to be, because copy imports 'inspect' which imports 'tokenize'.
+And 'copy' only needs 2 small functions out of 'inspect', but has to
+load all of 'tokenize', which makes it horribly slow.
+
+This module is designed to use tricky hacks in import rules, to avoid this
+overhead.
+"""
+
+
+####
+# These are the only 2 functions that 'copy' needs from 'inspect'
+# As you can see, they are quite trivial, and don't justify the
+# 40ms spent to import 'inspect' because it is importing 'tokenize'
+# These are copied verbatim from the python standard library.
+
+# ----------------------------------------------------------- class helpers
+def _searchbases(cls, accum):
+    # Simulate the "classic class" search order.
+    if cls in accum:
+        return
+    accum.append(cls)
+    for base in cls.__bases__:
+        _searchbases(base, accum)
+
+
+def getmro(cls):
+    "Return tuple of base classes (including cls) in method resolution order."
+    if hasattr(cls, "__mro__"):
+        return cls.__mro__
+    else:
+        result = []
+        _searchbases(cls, result)
+        return tuple(result)
+
+
+def import_copy_with_hacked_inspect():
+    """Import the 'copy' module with a hacked 'inspect' module"""
+    # We don't actually care about 'getmro' but we need to pass
+    # something in the list so that we get the direct module,
+    # rather than getting the base module
+    import sys
+
+    # Don't hack around if 'inspect' already exists
+    if 'inspect' in sys.modules:
+        import copy
+        return
+
+    mod = __import__('bzrlib.inspect_for_copy',
+                     globals(), locals(), ['getmro'])
+
+    sys.modules['inspect'] = mod
+    try:
+        import copy
+    finally:
+        del sys.modules['inspect']

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2006-07-29 08:46:23 +0000
+++ bzrlib/inter.py	2006-08-22 16:14:17 +0000
@@ -36,8 +36,8 @@
     the needs_read_lock and needs_write_lock decorators.)
     """
 
-    # _optimisers = set()
-    # Each concrete InterObject type should have its own optimisers set.
+    # _optimisers = list()
+    # Each concrete InterObject type should have its own optimisers list.
 
     def __init__(self, source, target):
         """Construct a default InterObject instance. Please use 'get'.
@@ -74,7 +74,7 @@
         If an optimised worker exists it will be used otherwise
         a default Inter worker instance will be created.
         """
-        for provider in klass._optimisers:
+        for provider in reversed(klass._optimisers):
             if provider.is_compatible(source, target):
                 return provider(source, target)
         return klass(source, target)
@@ -98,7 +98,7 @@
     @classmethod
     def register_optimiser(klass, optimiser):
         """Register an InterObject optimiser."""
-        klass._optimisers.add(optimiser)
+        klass._optimisers.append(optimiser)
 
     def unlock(self):
         """Release the locks on source and target."""

=== added file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lazy_import.py	2006-09-13 02:09:37 +0000
@@ -0,0 +1,379 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Functionality to create lazy evaluation objects.
+
+This includes waiting to import a module until it is actually used.
+
+Most commonly, the 'lazy_import' function is used to import other modules
+in an on-demand fashion. Typically use looks like:
+    from bzrlib.lazy_import import lazy_import
+    lazy_import(globals(), '''
+    from bzrlib import (
+        errors,
+        osutils,
+        branch,
+        )
+    import bzrlib.branch
+    ''')
+
+    Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
+    objects which will be replaced with a real object on first use.
+
+    In general, it is best to only load modules in this way. This is because
+    it isn't safe to pass these variables to other functions before they
+    have been replaced. This is especially true for constants, sometimes
+    true for classes or functions (when used as a factory, or you want
+    to inherit from them).
+"""
+
+import re
+import sys
+
+from bzrlib import (
+    errors,
+    )
+
+
+class ScopeReplacer(object):
+    """A lazy object that will replace itself in the appropriate scope.
+
+    This object sits, ready to create the real object the first time it is
+    needed.
+    """
+
+    __slots__ = ('_scope', '_factory', '_name')
+
+    def __init__(self, scope, factory, name):
+        """Create a temporary object in the specified scope.
+        Once used, a real object will be placed in the scope.
+
+        :param scope: The scope the object should appear in
+        :param factory: A callable that will create the real object.
+            It will be passed (self, scope, name)
+        :param name: The variable name in the given scope.
+        """
+        self._scope = scope
+        self._factory = factory
+        self._name = name
+        scope[name] = self
+
+    def _replace(self):
+        """Actually replace self with other in the given scope"""
+        name = object.__getattribute__(self, '_name')
+        try:
+            factory = object.__getattribute__(self, '_factory')
+            scope = object.__getattribute__(self, '_scope')
+        except AttributeError, e:
+            # Because ScopeReplacer objects only replace a single
+            # item, passing them to another variable before they are
+            # replaced would cause them to keep getting replaced
+            # (only they are replacing the wrong variable). So we
+            # make it forbidden, and try to give a good error.
+            raise errors.IllegalUseOfScopeReplacer(
+                name, msg="Object already cleaned up, did you assign it"
+                          " to another variable?",
+                extra=e)
+        obj = factory(self, scope, name)
+        scope[name] = obj
+        return obj
+
+    def _cleanup(self):
+        """Stop holding on to all the extra stuff"""
+        del self._factory
+        del self._scope
+        # We keep _name, so that we can report errors
+        # del self._name
+
+    def __getattribute__(self, attr):
+        _replace = object.__getattribute__(self, '_replace')
+        obj = _replace()
+        _cleanup = object.__getattribute__(self, '_cleanup')
+        _cleanup()
+        return getattr(obj, attr)
+
+    def __call__(self, *args, **kwargs):
+        _replace = object.__getattribute__(self, '_replace')
+        obj = _replace()
+        _cleanup = object.__getattribute__(self, '_cleanup')
+        _cleanup()
+        return obj(*args, **kwargs)
+
+
+class ImportReplacer(ScopeReplacer):
+    """This is designed to replace only a portion of an import list.
+
+    It will replace itself with a module, and then make children
+    entries also ImportReplacer objects.
+
+    At present, this only supports 'import foo.bar.baz' syntax.
+    """
+
+    # '_import_replacer_children' is intentionally a long semi-unique name
+    # that won't likely exist elsewhere. This allows us to detect an
+    # ImportReplacer object by using
+    #       object.__getattribute__(obj, '_import_replacer_children')
+    # We can't just use 'isinstance(obj, ImportReplacer)', because that
+    # accesses .__class__, which goes through __getattribute__, and triggers
+    # the replacement.
+    __slots__ = ('_import_replacer_children', '_member', '_module_path')
+
+    def __init__(self, scope, name, module_path, member=None, children={}):
+        """Upon request import 'module_path' as the name 'module_name'.
+        When imported, prepare children to also be imported.
+
+        :param scope: The scope that objects should be imported into.
+            Typically this is globals()
+        :param name: The variable name. Often this is the same as the 
+            module_path. 'bzrlib'
+        :param module_path: A list for the fully specified module path
+            ['bzrlib', 'foo', 'bar']
+        :param member: The member inside the module to import, often this is
+            None, indicating the module is being imported.
+        :param children: Children entries to be imported later.
+            This should be a map of children specifications.
+            {'foo':(['bzrlib', 'foo'], None, 
+                {'bar':(['bzrlib', 'foo', 'bar'], None {})})
+            }
+        Examples:
+            import foo => name='foo' module_path='foo',
+                          member=None, children={}
+            import foo.bar => name='foo' module_path='foo', member=None,
+                              children={'bar':(['foo', 'bar'], None, {}}
+            from foo import bar => name='bar' module_path='foo', member='bar'
+                                   children={}
+            from foo import bar, baz would get translated into 2 import
+            requests. On for 'name=bar' and one for 'name=baz'
+        """
+        if member is not None:
+            assert not children, \
+                'Cannot supply both a member and children'
+
+        self._import_replacer_children = children
+        self._member = member
+        self._module_path = module_path
+
+        # Indirecting through __class__ so that children can
+        # override _import (especially our instrumented version)
+        cls = object.__getattribute__(self, '__class__')
+        ScopeReplacer.__init__(self, scope=scope, name=name,
+                               factory=cls._import)
+
+    def _import(self, scope, name):
+        children = object.__getattribute__(self, '_import_replacer_children')
+        member = object.__getattribute__(self, '_member')
+        module_path = object.__getattribute__(self, '_module_path')
+        module_python_path = '.'.join(module_path)
+        if member is not None:
+            module = __import__(module_python_path, scope, scope, [member])
+            return getattr(module, member)
+        else:
+            module = __import__(module_python_path, scope, scope, [])
+            for path in module_path[1:]:
+                module = getattr(module, path)
+
+        # Prepare the children to be imported
+        for child_name, (child_path, child_member, grandchildren) in \
+                children.iteritems():
+            # Using self.__class__, so that children get children classes
+            # instantiated. (This helps with instrumented tests)
+            cls = object.__getattribute__(self, '__class__')
+            cls(module.__dict__, name=child_name,
+                module_path=child_path, member=child_member,
+                children=grandchildren)
+        return module
+
+
+class ImportProcessor(object):
+    """Convert text that users input into lazy import requests"""
+
+    # TODO: jam 20060912 This class is probably not strict enough about
+    #       what type of text it allows. For example, you can do:
+    #       import (foo, bar), which is not allowed by python.
+    #       For now, it should be supporting a superset of python import
+    #       syntax which is all we really care about.
+
+    __slots__ = ['imports', '_lazy_import_class']
+
+    def __init__(self, lazy_import_class=None):
+        self.imports = {}
+        if lazy_import_class is None:
+            self._lazy_import_class = ImportReplacer
+        else:
+            self._lazy_import_class = lazy_import_class
+
+    def lazy_import(self, scope, text):
+        """Convert the given text into a bunch of lazy import objects.
+
+        This takes a text string, which should be similar to normal python
+        import markup.
+        """
+        self._build_map(text)
+        self._convert_imports(scope)
+
+    def _convert_imports(self, scope):
+        # Now convert the map into a set of imports
+        for name, info in self.imports.iteritems():
+            self._lazy_import_class(scope, name=name, module_path=info[0],
+                                    member=info[1], children=info[2])
+
+    def _build_map(self, text):
+        """Take a string describing imports, and build up the internal map"""
+        for line in self._canonicalize_import_text(text):
+            if line.startswith('import '):
+                self._convert_import_str(line)
+            elif line.startswith('from '):
+                self._convert_from_str(line)
+            else:
+                raise errors.InvalidImportLine(line,
+                    "doesn't start with 'import ' or 'from '")
+
+    def _convert_import_str(self, import_str):
+        """This converts a import string into an import map.
+
+        This only understands 'import foo, foo.bar, foo.bar.baz as bing'
+
+        :param import_str: The import string to process
+        """
+        assert import_str.startswith('import ')
+        import_str = import_str[len('import '):]
+
+        for path in import_str.split(','):
+            path = path.strip()
+            if not path:
+                continue
+            as_hunks = path.split(' as ')
+            if len(as_hunks) == 2:
+                # We have 'as' so this is a different style of import
+                # 'import foo.bar.baz as bing' creates a local variable
+                # named 'bing' which points to 'foo.bar.baz'
+                name = as_hunks[1].strip()
+                module_path = as_hunks[0].strip().split('.')
+                if name in self.imports:
+                    raise errors.ImportNameCollision(name)
+                # No children available in 'import foo as bar'
+                self.imports[name] = (module_path, None, {})
+            else:
+                # Now we need to handle
+                module_path = path.split('.')
+                name = module_path[0]
+                if name not in self.imports:
+                    # This is a new import that we haven't seen before
+                    module_def = ([name], None, {})
+                    self.imports[name] = module_def
+                else:
+                    module_def = self.imports[name]
+
+                cur_path = [name]
+                cur = module_def[2]
+                for child in module_path[1:]:
+                    cur_path.append(child)
+                    if child in cur:
+                        cur = cur[child][2]
+                    else:
+                        next = (cur_path[:], None, {})
+                        cur[child] = next
+                        cur = next[2]
+
+    def _convert_from_str(self, from_str):
+        """This converts a 'from foo import bar' string into an import map.
+
+        :param from_str: The import string to process
+        """
+        assert from_str.startswith('from ')
+        from_str = from_str[len('from '):]
+
+        from_module, import_list = from_str.split(' import ')
+
+        from_module_path = from_module.split('.')
+
+        for path in import_list.split(','):
+            path = path.strip()
+            if not path:
+                continue
+            as_hunks = path.split(' as ')
+            if len(as_hunks) == 2:
+                # We have 'as' so this is a different style of import
+                # 'import foo.bar.baz as bing' creates a local variable
+                # named 'bing' which points to 'foo.bar.baz'
+                name = as_hunks[1].strip()
+                module = as_hunks[0].strip()
+            else:
+                name = module = path
+            if name in self.imports:
+                raise errors.ImportNameCollision(name)
+            self.imports[name] = (from_module_path, module, {})
+
+    def _canonicalize_import_text(self, text):
+        """Take a list of imports, and split it into regularized form.
+
+        This is meant to take regular import text, and convert it to
+        the forms that the rest of the converters prefer.
+        """
+        out = []
+        cur = None
+        continuing = False
+
+        for line in text.split('\n'):
+            line = line.strip()
+            loc = line.find('#')
+            if loc != -1:
+                line = line[:loc].strip()
+
+            if not line:
+                continue
+            if cur is not None:
+                if line.endswith(')'):
+                    out.append(cur + ' ' + line[:-1])
+                    cur = None
+                else:
+                    cur += ' ' + line
+            else:
+                if '(' in line and ')' not in line:
+                    cur = line.replace('(', '')
+                else:
+                    out.append(line.replace('(', '').replace(')', ''))
+        if cur is not None:
+            raise errors.InvalidImportLine(cur, 'Unmatched parenthesis')
+        return out
+
+
+def lazy_import(scope, text, lazy_import_class=None):
+    """Create lazy imports for all of the imports in text.
+
+    This is typically used as something like:
+    from bzrlib.lazy_import import lazy_import
+    lazy_import(globals(), '''
+    from bzrlib import (
+        foo,
+        bar,
+        baz,
+        )
+    import bzrlib.branch
+    import bzrlib.transport
+    ''')
+
+    Then 'foo, bar, baz' and 'bzrlib' will exist as lazy-loaded
+    objects which will be replaced with a real object on first use.
+
+    In general, it is best to only load modules in this way. This is
+    because other objects (functions/classes/variables) are frequently
+    used without accessing a member, which means we cannot tell they
+    have been used.
+    """
+    # This is just a helper around ImportProcessor.lazy_import
+    proc = ImportProcessor(lazy_import_class=lazy_import_class)
+    return proc.lazy_import(scope, text)

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-09-05 21:25:15 +0000
+++ bzrlib/lockdir.py	2006-09-09 00:52:19 +0000
@@ -114,7 +114,7 @@
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport
 from bzrlib.osutils import rand_chars
-from bzrlib.rio import RioWriter, read_stanza, Stanza
+from bzrlib.rio import read_stanza, Stanza
 
 # XXX: At the moment there is no consideration of thread safety on LockDir
 # objects.  This should perhaps be updated - e.g. if two threads try to take a
@@ -200,14 +200,13 @@
                 # After creating the lock directory, try again
                 self.transport.mkdir(tmpname)
 
-            sio = StringIO()
-            self._prepare_info(sio)
-            sio.seek(0)
-            # append will create a new file; we use append rather than put
-            # because we don't want to write to a temporary file and rename
-            # into place, because that's going to happen to the whole
-            # directory
-            self.transport.append_file(tmpname + self.__INFO_NAME, sio)
+            info_bytes = self._prepare_info()
+            # We use put_file_non_atomic because we just created a new unique
+            # directory so we don't have to worry about files existing there.
+            # We'll rename the whole directory into place to get atomic
+            # properties
+            self.transport.put_bytes_non_atomic(tmpname + self.__INFO_NAME,
+                                                info_bytes)
 
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
@@ -336,7 +335,7 @@
         except NoSuchFile, e:
             return None
 
-    def _prepare_info(self, outf):
+    def _prepare_info(self):
         """Write information about a pending lock to a temporary file.
         """
         import socket
@@ -348,7 +347,7 @@
                    nonce=self.nonce,
                    user=config.user_email(),
                    )
-        RioWriter(outf).write_stanza(s)
+        return s.to_string()
 
     def _parse_info(self, info_file):
         return read_stanza(info_file.readlines()).as_dict()

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-09-08 18:52:17 +0000
+++ bzrlib/osutils.py	2006-09-12 21:25:01 +0000
@@ -1052,7 +1052,7 @@
         _cached_user_encoding = locale.getpreferredencoding()
     except locale.Error, e:
         sys.stderr.write('bzr: warning: %s\n'
-                         '  Could not what text encoding to use.\n'
+                         '  Could not determine what text encoding to use.\n'
                          '  This error usually means your Python interpreter\n'
                          '  doesn\'t support the locale set by $LANG (%s)\n'
                          "  Continuing with ascii encoding.\n"

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-09-13 10:36:03 +0000
+++ bzrlib/repository.py	2006-09-14 04:10:40 +0000
@@ -21,7 +21,18 @@
 import time
 from unittest import TestSuite
 
-from bzrlib import bzrdir, check, delta, gpg, errors, xml5, ui, transactions, osutils
+from bzrlib import (
+    bzrdir, 
+    check, 
+    delta, 
+    gpg, 
+    errors, 
+    osutils,
+    transactions,
+    ui, 
+    xml5, 
+    xml6,
+    )
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.errors import InvalidRevisionId
 from bzrlib.graph import Graph
@@ -75,7 +86,7 @@
             "Mismatch between inventory revision" \
             " id and insertion revid (%r, %r)" % (inv.revision_id, revid)
         assert inv.root is not None
-        inv_text = xml5.serializer_v5.write_inventory_to_string(inv)
+        inv_text = self.serialise_inventory(inv)
         inv_sha1 = osutils.sha_string(inv_text)
         inv_vf = self.control_weaves.get_weave('inventory',
                                                self.get_transaction())
@@ -195,6 +206,7 @@
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
         self._warn_if_deprecated()
+        self._serializer = xml5.serializer_v5
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, 
@@ -399,10 +411,7 @@
         revision_ids. Each altered file-ids has the exact revision_ids that
         altered it listed explicitly.
         """
-        assert isinstance(self._format, (RepositoryFormat5,
-                                         RepositoryFormat6,
-                                         RepositoryFormat7,
-                                         RepositoryFormatKnit1)), \
+        assert self._serializer.support_altered_by_hack, \
             ("fileids_altered_by_revision_ids only supported for branches " 
              "which store inventory as unnested xml, not on %r" % self)
         selected_revision_ids = set(revision_ids)
@@ -450,10 +459,13 @@
         :param revision_id: The expected revision id of the inventory.
         :param xml: A serialised inventory.
         """
-        result = xml5.serializer_v5.read_inventory_from_string(xml)
+        result = self._serializer.read_inventory_from_string(xml)
         result.root.revision = revision_id
         return result
 
+    def serialise_inventory(self, inv):
+        return self._serializer.write_inventory_to_string(inv)
+
     @needs_read_lock
     def get_inventory_xml(self, revision_id):
         """Get inventory XML as a file object."""
@@ -989,6 +1001,52 @@
         return self._get_revision_vf().get_parents(revision_id)
 
 
+class KnitRepository2(KnitRepository):
+    """"""
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store,
+                 control_store, text_store):
+        KnitRepository.__init__(self, _format, a_bzrdir, control_files,
+                              _revision_store, control_store, text_store)
+        self._serializer = xml6.serializer_v6
+
+    def deserialise_inventory(self, revision_id, xml):
+        """Transform the xml into an inventory object. 
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        result = self._serializer.read_inventory_from_string(xml)
+        assert result.root.revision is not None
+        return result
+
+    def serialise_inventory(self, inv):
+        """Transform the inventory object into XML text.
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        assert inv.revision_id is not None
+        assert inv.root.revision is not None
+        return KnitRepository.serialise_inventory(self, inv)
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None, 
+                           timezone=None, committer=None, revprops=None, 
+                           revision_id=None):
+        """Obtain a CommitBuilder for this repository.
+        
+        :param branch: Branch to commit to.
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        """
+        return RootCommitBuilder(self, parents, config, timestamp, timezone,
+                                 committer, revprops, revision_id)
+
+
 class RepositoryFormat(object):
     """A repository format.
 
@@ -1119,6 +1177,9 @@
         """
         return True
 
+    def check_conversion_target(self, target_format):
+        raise NotImplementedError(self.check_conversion_target)
+
     def open(self, a_bzrdir, _found=False):
         """Return an instance of this format for the bzrdir a_bzrdir.
         
@@ -1143,6 +1204,8 @@
 class PreSplitOutRepositoryFormat(RepositoryFormat):
     """Base class for the pre split out repository formats."""
 
+    rich_root_data = False
+
     def initialize(self, a_bzrdir, shared=False, _internal=False):
         """Create a weave repository.
         
@@ -1212,6 +1275,9 @@
                                   control_store=control_store,
                                   text_store=text_store)
 
+    def check_conversion_target(self, target_format):
+        pass
+
 
 class RepositoryFormat4(PreSplitOutRepositoryFormat):
     """Bzr repository format 4.
@@ -1328,6 +1394,8 @@
 class MetaDirRepositoryFormat(RepositoryFormat):
     """Common base class for the new repositories using the metadir layout."""
 
+    rich_root_data = False
+
     def __init__(self):
         super(MetaDirRepositoryFormat, self).__init__()
         self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
@@ -1385,6 +1453,9 @@
         """See RepositoryFormat.get_format_description()."""
         return "Weave repository format 7"
 
+    def check_conversion_target(self, target_format):
+        pass
+
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         return self._get_text_rev_store(repo_transport,
@@ -1449,8 +1520,8 @@
                                  text_store=text_store)
 
 
-class RepositoryFormatKnit1(MetaDirRepositoryFormat):
-    """Bzr repository knit format 1.
+class RepositoryFormatKnit(MetaDirRepositoryFormat):
+    """Bzr repository knit format (generalized). 
 
     This repository format has:
      - knits for file texts and inventory
@@ -1461,8 +1532,6 @@
      - an optional 'shared-storage' flag
      - an optional 'no-working-trees' flag
      - a LockDir lock
-
-    This format was introduced in bzr 0.8.
     """
 
     def _get_control_store(self, repo_transport, control_files):
@@ -1475,14 +1544,6 @@
             versionedfile_kwargs={'factory':KnitPlainFactory()},
             )
 
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar-NG Knit Repository Format 1"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Knit repository format 1"
-
     def _get_revision_store(self, repo_transport, control_files):
         """See RepositoryFormat._get_revision_store()."""
         from bzrlib.store.revision.knit import KnitRevisionStore
@@ -1561,11 +1622,98 @@
                               text_store=text_store)
 
 
+class RepositoryFormatKnit1(RepositoryFormatKnit):
+    """Bzr repository knit format 1.
+
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+
+    This format was introduced in bzr 0.8.
+    """
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar-NG Knit Repository Format 1"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 1"
+
+    def check_conversion_target(self, target_format):
+        pass
+
+
+class RepositoryFormatKnit2(RepositoryFormatKnit):
+    """Bzr repository knit format 2.
+
+    THIS FORMAT IS EXPERIMENTAL
+    This repository format has:
+     - knits for file texts and inventory
+     - hash subdirectory based stores.
+     - knits for revisions and signatures
+     - TextStores for revisions and signatures.
+     - a format marker of its own
+     - an optional 'shared-storage' flag
+     - an optional 'no-working-trees' flag
+     - a LockDir lock
+     - Support for recording full info about the tree root
+
+    """
+    
+    rich_root_data = True
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar Knit Repository Format 2\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Knit repository format 2"
+
+    def check_conversion_target(self, target_format):
+        if not target_format.rich_root_data:
+            raise errors.BadConversionTarget(
+                'Does not support rich root data.', target_format)
+
+    def open(self, a_bzrdir, _found=False, _override_transport=None):
+        """See RepositoryFormat.open().
+        
+        :param _override_transport: INTERNAL USE ONLY. Allows opening the
+                                    repository at a slightly different url
+                                    than normal. I.e. during 'upgrade'.
+        """
+        if not _found:
+            format = RepositoryFormat.find_format(a_bzrdir)
+            assert format.__class__ ==  self.__class__
+        if _override_transport is not None:
+            repo_transport = _override_transport
+        else:
+            repo_transport = a_bzrdir.get_repository_transport(None)
+        control_files = LockableFiles(repo_transport, 'lock', LockDir)
+        text_store = self._get_text_store(repo_transport, control_files)
+        control_store = self._get_control_store(repo_transport, control_files)
+        _revision_store = self._get_revision_store(repo_transport, control_files)
+        return KnitRepository2(_format=self,
+                               a_bzrdir=a_bzrdir,
+                               control_files=control_files,
+                               _revision_store=_revision_store,
+                               control_store=control_store,
+                               text_store=text_store)
+
+
+
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
 RepositoryFormat.register_format(RepositoryFormat7())
 _default_format = RepositoryFormatKnit1()
 RepositoryFormat.register_format(_default_format)
+RepositoryFormat.register_format(RepositoryFormatKnit2())
 RepositoryFormat.set_default_format(_default_format)
 _legacy_formats = [RepositoryFormat4(),
                    RepositoryFormat5(),
@@ -1584,34 +1732,12 @@
     InterRepository.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterRepository types."""
 
-    @needs_write_lock
     def copy_content(self, revision_id=None, basis=None):
-        """Make a complete copy of the content in self into destination.
-        
-        This is a destructive operation! Do not use it on existing 
-        repositories.
-
-        :param revision_id: Only copy the content needed to construct
-                            revision_id and its parents.
-        :param basis: Copy the needed data preferentially from basis.
-        """
-        try:
-            self.target.set_make_working_trees(self.source.make_working_trees())
-        except NotImplementedError:
-            pass
-        # grab the basis available data
-        if basis is not None:
-            self.target.fetch(basis, revision_id=revision_id)
-        # but don't bother fetching if we have the needed data now.
-        if (revision_id not in (None, NULL_REVISION) and 
-            self.target.has_revision(revision_id)):
-            return
-        self.target.fetch(self.source, revision_id=revision_id)
-
-    @needs_write_lock
+        raise NotImplementedError(self.copy_content)
+
     def fetch(self, revision_id=None, pb=None):
         """Fetch the content required to construct revision_id.
 
@@ -1625,15 +1751,8 @@
         Returns the copied revision count and the failed revisions in a tuple:
         (copied, failures).
         """
-        from bzrlib.fetch import GenericRepoFetcher
-        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
-               self.source, self.source._format, self.target, self.target._format)
-        f = GenericRepoFetcher(to_repository=self.target,
-                               from_repository=self.source,
-                               last_revision=revision_id,
-                               pb=pb)
-        return f.count_copied, f.failed_revisions
-
+        raise NotImplementedError(self.fetch)
+   
     @needs_read_lock
     def missing_revision_ids(self, revision_id=None):
         """Return the revision ids that source has that target does not.
@@ -1658,7 +1777,65 @@
         return [rev_id for rev_id in source_ids if rev_id in result_set]
 
 
-class InterWeaveRepo(InterRepository):
+class InterSameDataRepository(InterRepository):
+    """Code for converting between repositories that represent the same data.
+    
+    Data format and model must match for this to work.
+    """
+
+    _matching_repo_format = RepositoryFormat4()
+    """Repository format for testing with."""
+
+    @staticmethod
+    def is_compatible(source, target):
+        if not isinstance(source, Repository):
+            return False
+        if not isinstance(target, Repository):
+            return False
+        if source._format.rich_root_data == target._format.rich_root_data:
+            return True
+        else:
+            return False
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+
+        :param revision_id: Only copy the content needed to construct
+                            revision_id and its parents.
+        :param basis: Copy the needed data preferentially from basis.
+        """
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except NotImplementedError:
+            pass
+        # grab the basis available data
+        if basis is not None:
+            self.target.fetch(basis, revision_id=revision_id)
+        # but don't bother fetching if we have the needed data now.
+        if (revision_id not in (None, NULL_REVISION) and 
+            self.target.has_revision(revision_id)):
+            return
+        self.target.fetch(self.source, revision_id=revision_id)
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import GenericRepoFetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, 
+               self.target._format)
+        f = GenericRepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               pb=pb)
+        return f.count_copied, f.failed_revisions
+
+
+class InterWeaveRepo(InterSameDataRepository):
     """Optimised code paths between Weave based repositories."""
 
     _matching_repo_format = RepositoryFormat7()
@@ -1776,7 +1953,7 @@
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
 
-class InterKnitRepo(InterRepository):
+class InterKnitRepo(InterSameDataRepository):
     """Optimised code paths between Knit based repositories."""
 
     _matching_repo_format = RepositoryFormatKnit1()
@@ -1838,8 +2015,89 @@
             # that against the revision records.
             return self.source._eliminate_revisions_not_present(required_topo_revisions)
 
+
+class InterModel1and2(InterRepository):
+
+    _matching_repo_format = None
+
+    @staticmethod
+    def is_compatible(source, target):
+        if not isinstance(source, Repository):
+            return False
+        if not isinstance(target, Repository):
+            return False
+        if not source._format.rich_root_data and target._format.rich_root_data:
+            return True
+        else:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import Model1toKnit2Fetcher
+        f = Model1toKnit2Fetcher(to_repository=self.target,
+                                 from_repository=self.source,
+                                 last_revision=revision_id,
+                                 pb=pb)
+        return f.count_copied, f.failed_revisions
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None, basis=None):
+        """Make a complete copy of the content in self into destination.
+        
+        This is a destructive operation! Do not use it on existing 
+        repositories.
+
+        :param revision_id: Only copy the content needed to construct
+                            revision_id and its parents.
+        :param basis: Copy the needed data preferentially from basis.
+        """
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except NotImplementedError:
+            pass
+        # grab the basis available data
+        if basis is not None:
+            self.target.fetch(basis, revision_id=revision_id)
+        # but don't bother fetching if we have the needed data now.
+        if (revision_id not in (None, NULL_REVISION) and 
+            self.target.has_revision(revision_id)):
+            return
+        self.target.fetch(self.source, revision_id=revision_id)
+
+
+class InterKnit1and2(InterKnitRepo):
+
+    _matching_repo_format = None
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with Knit1 source and Knit2 target"""
+        try:
+            return (isinstance(source._format, (RepositoryFormatKnit1)) and
+                    isinstance(target._format, (RepositoryFormatKnit2)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, pb=None):
+        """See InterRepository.fetch()."""
+        from bzrlib.fetch import Knit1to2Fetcher
+        mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
+               self.source, self.source._format, self.target, 
+               self.target._format)
+        f = Knit1to2Fetcher(to_repository=self.target,
+                            from_repository=self.source,
+                            last_revision=revision_id,
+                            pb=pb)
+        return f.count_copied, f.failed_revisions
+
+
+InterRepository.register_optimiser(InterSameDataRepository)
 InterRepository.register_optimiser(InterWeaveRepo)
 InterRepository.register_optimiser(InterKnitRepo)
+InterRepository.register_optimiser(InterModel1and2)
+InterRepository.register_optimiser(InterKnit1and2)
 
 
 class RepositoryTestProviderAdapter(object):
@@ -1910,16 +2168,21 @@
         # default format.
         # XXX: robertc 20060220 reinstate this when there are two supported
         # formats which do not have an optimal code path between them.
-        result.append((InterRepository,
-                       RepositoryFormat6(),
-                       RepositoryFormatKnit1()))
+        #result.append((InterRepository,
+        #               RepositoryFormat6(),
+        #               RepositoryFormatKnit1()))
         for optimiser in InterRepository._optimisers:
-            result.append((optimiser,
-                           optimiser._matching_repo_format,
-                           optimiser._matching_repo_format
-                           ))
+            if optimiser._matching_repo_format is not None:
+                result.append((optimiser,
+                               optimiser._matching_repo_format,
+                               optimiser._matching_repo_format
+                               ))
         # if there are specific combinations we want to use, we can add them 
         # here.
+        result.append((InterModel1and2, RepositoryFormat5(),
+                       RepositoryFormatKnit2()))
+        result.append((InterKnit1and2, RepositoryFormatKnit1(),
+                       RepositoryFormatKnit2()))
         return result
 
 
@@ -1952,6 +2215,7 @@
         self.step('Moving repository to repository.backup')
         self.repo_dir.transport.move('repository', 'repository.backup')
         backup_transport =  self.repo_dir.transport.clone('repository.backup')
+        repo._format.check_conversion_target(self.target_format)
         self.source_repo = repo._format.open(self.repo_dir,
             _found=True,
             _override_transport=backup_transport)
@@ -2182,6 +2446,41 @@
     record_root_entry = True
 
 
+class RootCommitBuilder(CommitBuilder):
+    """This commitbuilder actually records the root id"""
+    
+    record_root_entry = True
+
+    def record_entry_contents(self, ie, parent_invs, path, tree):
+        """Record the content of ie from tree into the commit if needed.
+
+        Side effect: sets ie.revision when unchanged
+
+        :param ie: An inventory entry present in the commit.
+        :param parent_invs: The inventories of the parent revisions of the
+            commit.
+        :param path: The path the entry is at in the tree.
+        :param tree: The tree which contains this entry and should be used to 
+        obtain content.
+        """
+        assert self.new_inventory.root is not None or ie.parent_id is None
+        self.new_inventory.add(ie)
+
+        # ie.revision is always None if the InventoryEntry is considered
+        # for committing. ie.snapshot will record the correct revision 
+        # which may be the sole parent if it is untouched.
+        if ie.revision is not None:
+            return
+
+        previous_entries = ie.find_previous_heads(
+            parent_invs,
+            self.repository.weave_store,
+            self.repository.get_transaction())
+        # we are creating a new revision for ie in the history store
+        # and inventory.
+        ie.snapshot(self._new_revision_id, path, previous_entries, tree, self)
+
+
 _unescape_map = {
     'apos':"'",
     'quot':'"',

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2006-08-28 16:28:10 +0000
+++ bzrlib/revisionspec.py	2006-09-03 15:58:15 +0000
@@ -224,6 +224,13 @@
         """
         return True
 
+    def get_branch(self):
+        """When the revision specifier contains a branch location, return it.
+        
+        Otherwise, return None.
+        """
+        return None
+
 
 # private API
 
@@ -273,6 +280,12 @@
     def needs_branch(self):
         return self.spec.find(':') == -1
 
+    def get_branch(self):
+        if self.spec.find(':') == -1:
+            return None
+        else:
+            return self.spec[self.spec.find(':')+1:]
+
 # Old compatibility 
 RevisionSpec_int = RevisionSpec_revno
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-13 10:36:03 +0000
+++ bzrlib/tests/__init__.py	2006-09-14 04:10:40 +0000
@@ -898,6 +898,9 @@
                 osutils.set_or_unset_env(env_var, value)
 
         bzr_path = os.path.dirname(os.path.dirname(bzrlib.__file__))+'/bzr'
+        if not os.path.isfile(bzr_path):
+            # We are probably installed. Assume sys.argv is the right file
+            bzr_path = sys.argv[0]
         args = list(args)
 
         try:
@@ -920,7 +923,11 @@
         retcode = process.wait()
         supplied_retcode = kwargs.get('retcode', 0)
         if supplied_retcode is not None:
-            assert supplied_retcode == retcode
+            if supplied_retcode != retcode:
+                mutter('Output of bzr %s:\n%s', args, out)
+                mutter('Error for bzr %s:\n%s', args, err)
+                self.fail('Command bzr %s failed with retcode %s != %s'
+                          % (args, supplied_retcode, retcode))
         return [out, err]
 
     def check_inventory_shape(self, inv, shape):
@@ -1424,6 +1431,7 @@
                    'bzrlib.tests.test_ignores',
                    'bzrlib.tests.test_inv',
                    'bzrlib.tests.test_knit',
+                   'bzrlib.tests.test_lazy_import',
                    'bzrlib.tests.test_lockdir',
                    'bzrlib.tests.test_lockable_files',
                    'bzrlib.tests.test_log',

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2006-09-11 06:07:02 +0000
@@ -31,6 +31,8 @@
         def bzr(*args, **kwargs):
             return self.run_bzr(*args, **kwargs)[0]
 
+        os.mkdir('branch')
+        os.chdir('branch')
         bzr('init')
         open('a', 'wb').write('foo\n')
         bzr('add', 'a')
@@ -51,4 +53,11 @@
         rev_id = bzr('revision-history').strip().split('\n')[-1]
 
         self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
-
+        
+        os.chdir('..')
+        
+        self.assertEquals(bzr('cat', 'branch/a', '-r', 'revno:1:branch'),
+                          'foo\n')
+        bzr('cat', 'a', retcode=3)
+        bzr('cat', 'a', '-r', 'revno:1:branch-that-does-not-exist', retcode=3)
+        

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-08-14 15:36:22 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-08-24 18:58:29 +0000
@@ -21,6 +21,7 @@
 import sys
 
 import bzrlib
+from bzrlib import repository
 from bzrlib.osutils import format_date
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
@@ -133,7 +134,9 @@
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
         branch1.bzrdir.sprout('bound')
-        bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
+        knit1_format = bzrlib.bzrdir.BzrDirMetaFormat1()
+        knit1_format.repository_format = repository.RepositoryFormatKnit1()
+        bzrlib.upgrade.upgrade('bound', knit1_format)
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
         bound_tree = branch3.bzrdir.open_workingtree()
@@ -186,7 +189,7 @@
 
         # Checkout standalone (same as above, but does not have parent set)
         old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(knit1_format)
         branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout')
         bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
         branch4.bind(branch1)

=== modified file 'bzrlib/tests/blackbox/test_locale.py'
--- bzrlib/tests/blackbox/test_locale.py	2006-09-09 15:39:00 +0000
+++ bzrlib/tests/blackbox/test_locale.py	2006-09-12 21:25:01 +0000
@@ -65,7 +65,7 @@
         # as the first part of the string. It may be a little tempermental
         self.assertEqualDiff("""\
 bzr: warning: unsupported locale setting
-  Could not what text encoding to use.
+  Could not determine what text encoding to use.
   This error usually means your Python interpreter
   doesn't support the locale set by $LANG (BOGUS)
   Continuing with ascii encoding.

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/blackbox/test_log.py	2006-08-06 17:31:40 +0000
@@ -89,6 +89,25 @@
         log = self.runbzr("log -r 1..3")[0]
         self.assertEquals(self.full_log, log)
 
+    def test_log_revno_n_path(self):
+        os.mkdir('branch1')
+        os.chdir('branch1')
+        self._prepare()
+        os.chdir('..')
+        os.mkdir('branch2')
+        os.chdir('branch2')
+        self._prepare()
+        os.chdir('..')
+        log = self.runbzr("log -r revno:2:branch1..revno:3:branch2",
+                          retcode=3)[0]
+        log = self.runbzr("log -r revno:1:branch2..revno:3:branch2")[0]
+        self.assertEquals(self.full_log, log)
+        log = self.runbzr("log -r revno:1:branch2")[0]
+        self.assertTrue('revno: 1\n' in log)
+        self.assertTrue('revno: 2\n' not in log)
+        self.assertTrue('branch nick: branch2\n' in log)
+        self.assertTrue('branch nick: branch1\n' not in log)
+        
 
 class TestLogMerges(ExternalBase):
 

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2006-09-11 09:34:51 +0000
@@ -52,6 +52,7 @@
         os.mkdir('a')
         os.chdir('a')
         self.example_branch()
+        ancestor = Branch.open('.').revno()
         os.chdir('..')
         self.runbzr('branch a b')
         os.chdir('b')
@@ -79,6 +80,17 @@
         b_tip = b.last_revision()
         self.failUnless(a.branch.repository.has_revision(b_tip))
         self.assertEqual([a_tip, b_tip], a.get_parent_ids())
+        self.runbzr('revert --no-backup')
+        out, err = self.runbzr('merge -r revno:1:./hello', retcode=3)
+        self.assertTrue("Not a branch" in err)
+        self.runbzr('merge -r revno:%d:./..revno:%d:../b'
+                    %(ancestor,b.revno()))
+        self.assertEquals(a.pending_merges(), [b.last_revision()])
+        self.check_file_contents('goodbye', 'quux')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge -r revno:%d:../b'%b.revno())
+        self.assertEquals(a.pending_merges(),
+                          [b.last_revision()])
         a_tip = a.commit('merged')
         self.runbzr('merge ../b -r last:1')
         self.assertEqual([a_tip], a.get_parent_ids())

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2006-08-24 18:37:24 +0000
@@ -52,7 +52,8 @@
         t = bzrdir.BzrDir.create_standalone_workingtree('.')
         # an empty inventory with no revision will trigger reconciliation.
         repo = t.branch.repository
-        inv = Inventory()
+        inv = Inventory(revision_id='missing')
+        inv.root.revision='missing'
         repo.add_inventory('missing', inv, [])
         (out, err) = self.run_bzr_captured(['reconcile'])
         self.assertEqualDiff(out, "Reconciling repository %s\n"

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2006-09-10 23:56:57 +0000
@@ -405,6 +405,18 @@
         os.mkdir('e')
         checkout_e = branch_a.create_checkout('e')
 
+    def test_create_anonymous_lightweight_checkout(self):
+        """A checkout from a readonly branch should succeed."""
+        tree_a = self.make_branch_and_tree('a')
+        rev_id = tree_a.commit('put some content in the branch')
+        source_branch = bzrlib.branch.Branch.open(
+            'readonly+' + tree_a.bzrdir.root_transport.base)
+        # sanity check that the test will be valid
+        self.assertRaises((errors.LockError, errors.TransportNotPossible),
+            source_branch.lock_write)
+        checkout = source_branch.create_checkout('c', lightweight=True)
+        self.assertEqual(rev_id, checkout.last_revision())
+
 
 class ChrootedTests(TestCaseWithBranch):
     """A support class that provides readonly urls outside the local namespace.

=== modified file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2006-09-09 18:52:57 +0000
@@ -188,6 +188,21 @@
         to_repo = self.make_to_repository('to')
         to_repo.fetch(from_tree.branch.repository, from_tree.get_parent_ids()[0])
 
+    def test_fetch_no_inventory_revision(self):
+        """Old inventories lack revision_ids, so simulate this"""
+        from_tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/filename'])
+        from_tree.add('filename', 'funky-chars<>%&;"\'')
+        from_tree.commit('commit filename')
+        old_deserialise = from_tree.branch.repository.deserialise_inventory
+        def deserialise(revision_id, text):
+            inventory = old_deserialise(revision_id, text)
+            inventory.revision_id = None
+            return inventory
+        from_tree.branch.repository.deserialise_inventory = deserialise
+        to_repo = self.make_to_repository('to')
+        to_repo.fetch(from_tree.branch.repository, from_tree.last_revision())
+
 
 class TestCaseWithComplexRepository(TestCaseWithInterRepository):
 
@@ -248,7 +263,8 @@
         # repository.
 
         # 'ghost' is a ghost in missing_ghost and not in with_ghost_rev
-        inv = Inventory()
+        inv = Inventory(revision_id='ghost')
+        inv.root.revision = 'ghost'
         repo = self.make_repository('with_ghost_rev')
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
@@ -261,6 +277,8 @@
         repo.add_revision('ghost', rev)
          
         repo = self.make_to_repository('missing_ghost')
+        inv = Inventory(revision_id='with_ghost')
+        inv.root.revision = 'with_ghost'
         sha1 = repo.add_inventory('with_ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,

=== modified file 'bzrlib/tests/repository_implementations/test_commit_builder.py'
--- bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-05 23:01:33 +0000
+++ bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-09 18:52:57 +0000
@@ -19,6 +19,7 @@
 from bzrlib import inventory
 from bzrlib.errors import UnsupportedOperation
 from bzrlib.repository import CommitBuilder
+from bzrlib import tests
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 
 
@@ -33,7 +34,7 @@
         if builder.record_root_entry is True:
             ie = tree.inventory.root
             parent_tree = tree.branch.repository.revision_tree(None)
-            parent_invs = [parent_tree.inventory]
+            parent_invs = []
             builder.record_entry_contents(ie, parent_invs, '', tree)
 
     def test_finish_inventory(self):
@@ -70,6 +71,8 @@
     def test_commit_without_root(self):
         """This should cause a deprecation warning, not an assertion failure"""
         tree = self.make_branch_and_tree(".")
+        if tree.branch.repository._format.rich_root_data:
+            raise tests.TestSkipped('Format requires root')
         self.build_tree(['foo'])
         tree.add('foo', 'foo-id')
         entry = tree.inventory['foo-id']

=== modified file 'bzrlib/tests/repository_implementations/test_fileid_involved.py'
--- bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-09 18:52:57 +0000
@@ -173,13 +173,21 @@
             self.branch.repository.fileids_altered_by_revision_ids(
                 ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
 
+    def fileids_altered_by_revision_ids(self, revision_ids):
+        """This is a wrapper to strip TREE_ROOT if it occurs"""
+        repo = self.branch.repository
+        result = repo.fileids_altered_by_revision_ids(revision_ids)
+        if 'TREE_ROOT' in result:
+            del result['TREE_ROOT']
+        return result
+
     def test_fileids_altered_by_revision_ids(self):
         self.assertEqual(
             {'a-file-id-2006-01-01-abcd':set(['rev-A']),
              'b-file-id-2006-01-01-defg': set(['rev-A']),
              'c-funky<file-id> quiji%bo': set(['rev-A']),
              }, 
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-A"]))
+            self.fileids_altered_by_revision_ids(["rev-A"]))
         self.assertEqual(
             {'a-file-id-2006-01-01-abcd':set(['rev-B'])
              }, 

=== modified file 'bzrlib/tests/repository_implementations/test_reconcile.py'
--- bzrlib/tests/repository_implementations/test_reconcile.py	2006-08-04 04:41:02 +0000
+++ bzrlib/tests/repository_implementations/test_reconcile.py	2006-08-23 13:15:45 +0000
@@ -56,7 +56,8 @@
         t = get_transport(self.get_url())
         # an empty inventory with no revision for testing with.
         repo = self.make_repository('inventory_without_revision')
-        inv = Inventory()
+        inv = Inventory(revision_id='missing')
+        inv.root.revision = 'missing'
         repo.add_inventory('missing', inv, [])
 
         # an empty inventory with no revision for testing with.
@@ -64,8 +65,9 @@
         # that all the cached data is correctly converted into ghost links
         # and the referenced inventory still cleaned.
         repo = self.make_repository('inventory_without_revision_and_ghost')
-        inv = Inventory()
         repo.add_inventory('missing', inv, [])
+        inv = Inventory(revision_id='references_missing')
+        inv.root.revision = 'references_missing'
         sha1 = repo.add_inventory('references_missing', inv, ['missing'])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -79,6 +81,8 @@
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
         repo = self.make_repository('inventory_one_ghost')
+        inv = Inventory(revision_id='ghost')
+        inv.root.revision = 'ghost'
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -92,6 +96,8 @@
         # a inventory with a ghost that can be corrected now.
         t.copy_tree('inventory_one_ghost', 'inventory_ghost_present')
         repo = bzrlib.repository.Repository.open('inventory_ghost_present')
+        inv = Inventory(revision_id='the_ghost')
+        inv.root.revision = 'the_ghost'
         sha1 = repo.add_inventory('the_ghost', inv, [])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -265,7 +271,8 @@
 
         # now setup the wrong-first parent case
         repo = tree.branch.repository
-        inv = Inventory()
+        inv = Inventory(revision_id='wrong-first-parent')
+        inv.root.revision = 'wrong-first-parent'
         sha1 = repo.add_inventory('wrong-first-parent', inv, ['2', '1'])
         rev = Revision(timestamp=0,
                        timezone=None,
@@ -278,7 +285,8 @@
 
         # now setup the wrong-secondary parent case
         repo = repo_secondary
-        inv = Inventory()
+        inv = Inventory(revision_id='wrong-secondary-parent')
+        inv.root.revision = 'wrong-secondary-parent'
         sha1 = repo.add_inventory('wrong-secondary-parent', inv, ['1', '3', '2'])
         rev = Revision(timestamp=0,
                        timezone=None,

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-09-09 18:52:57 +0000
@@ -162,9 +162,36 @@
         tree_a.commit('rev1', rev_id='rev1')
         # fetch with a default limit (grab everything)
         repo = bzrdir.BzrDir.create_repository(self.get_url('b'))
-        repo.fetch(tree_a.branch.repository,
-                   revision_id=None,
-                   pb=bzrlib.progress.DummyProgress())
+        if (tree_a.branch.repository._format.rich_root_data and not
+            repo._format.rich_root_data):
+            raise TestSkipped('Cannot fetch from model2 to model1')
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+
+    def test_fetch_knit2(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        # fetch with a default limit (grab everything)
+        f = bzrdir.BzrDirMetaFormat1()
+        f._repository_format = repository.RepositoryFormatKnit2()
+        os.mkdir('b')
+        b_bzrdir = f.initialize(self.get_url('b'))
+        repo = b_bzrdir.create_repository()
+        repo.fetch(tree_a.branch.repository,
+                   revision_id=None,
+                   pb=bzrlib.progress.DummyProgress())
+        rev1_tree = repo.revision_tree('rev1')
+        lines = rev1_tree.get_file_lines(rev1_tree.inventory.root.file_id)
+        self.assertEqual([], lines)
+        b_branch = b_bzrdir.create_branch()
+        b_branch.pull(tree_a.branch)
+        tree_b = b_bzrdir.create_workingtree()
+        tree_b.commit('no change', rev_id='rev2')
+        rev2_tree = repo.revision_tree('rev2')
+        self.assertEqual('rev1', rev2_tree.inventory.root.revision)
 
     def test_get_revision_delta(self):
         tree_a = self.make_branch_and_tree('a')
@@ -240,6 +267,8 @@
         except errors.UpToDateFormat:
             # this is in the most current format already.
             return
+        except errors.BadConversionTarget, e:
+            raise TestSkipped(str(e))
         wt = WorkingTree.open(wt.basedir)
         new_signature = wt.branch.repository.get_signature_text('A')
         self.assertEqual(old_signature, new_signature)
@@ -320,6 +349,56 @@
         rev_tree = tree.branch.repository.revision_tree(tree.get_parent_ids()[0])
         self.assertEqual('rev_id', rev_tree.inventory.root.revision)
 
+    def test_create_basis_inventory(self):
+        # Needs testing here because differences between repo and working tree
+        # basis inventory formats can lead to bugs.
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv = t.basis_tree().inventory
+        self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.repository.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        t._control_files.get_utf8('basis-inventory-cache')
+
+        basis_inv_txt = t.read_basis_inventory()
+        basis_inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(basis_inv_txt)
+        self.assertEquals('r2', basis_inv.revision_id)
+        store_inv = b.repository.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+    def test_upgrade_from_format4(self):
+        from bzrlib.tests.test_upgrade import _upgrade_dir_template
+        if self.repository_format.__class__ == repository.RepositoryFormat4:
+            raise TestSkipped('Cannot convert format-4 to itself')
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        format = self.repository_format._matchingbzrdir
+        try:
+            format.repository_format = self.repository_format
+        except AttributeError:
+            pass
+        upgrade('.', format)
+
+    def test_pointless_commit(self):
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.PointlessCommit, tree.commit, 'pointless',
+                          allow_pointless=False)
+        tree.commit('pointless', allow_pointless=True)
+
 
 class TestCaseWithComplexRepository(TestCaseWithRepository):
 
@@ -434,7 +513,8 @@
         # a inventory with no parents and the revision has parents..
         # i.e. a ghost.
         repo = self.make_repository('inventory_with_unnecessary_ghost')
-        inv = Inventory()
+        inv = Inventory(revision_id = 'ghost')
+        inv.root.revision = 'ghost'
         sha1 = repo.add_inventory('ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,
@@ -445,6 +525,8 @@
         rev.parent_ids = ['the_ghost']
         repo.add_revision('ghost', rev)
          
+        inv = Inventory(revision_id = 'the_ghost')
+        inv.root.revision = 'the_ghost'
         sha1 = repo.add_inventory('the_ghost', inv, [])
         rev = bzrlib.revision.Revision(timestamp=0,
                                        timezone=None,

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-07-14 19:25:24 +0000
+++ bzrlib/tests/test_http.py	2006-09-12 15:28:02 +0000
@@ -19,6 +19,8 @@
 
 # TODO: Should be renamed to bzrlib.transport.http.tests?
 
+import socket
+
 import bzrlib
 from bzrlib.errors import DependencyNotPresent
 from bzrlib.tests import TestCase, TestSkipped
@@ -135,6 +137,14 @@
         TestCaseWithWebserver.setUp(self)
         self._prep_tree()
 
+    def test_has_on_bogus_host(self):
+        import urllib2
+        # Get a random address, so that we can be sure there is no
+        # http handler there.
+        s = socket.socket()
+        s.bind(('localhost', 0))
+        t = self._transport('http://%s:%s/' % s.getsockname())
+        self.assertRaises(urllib2.URLError, t.has, 'foo/bar')
 
 
 class TestHttpConnections_pycurl(TestCaseWithWebserver, TestHttpMixins):

=== added file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_lazy_import.py	2006-09-13 02:16:02 +0000
@@ -0,0 +1,1043 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test the lazy_import functionality."""
+
+import os
+import sys
+
+from bzrlib import (
+    errors,
+    lazy_import,
+    osutils,
+    )
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+class InstrumentedReplacer(lazy_import.ScopeReplacer):
+    """Track what actions are done"""
+
+    @staticmethod
+    def use_actions(actions):
+        InstrumentedReplacer.actions = actions
+
+    def _replace(self):
+        InstrumentedReplacer.actions.append('_replace')
+        return lazy_import.ScopeReplacer._replace(self)
+
+    def __getattribute__(self, attr):
+        InstrumentedReplacer.actions.append(('__getattribute__', attr))
+        return lazy_import.ScopeReplacer.__getattribute__(self, attr)
+
+    def __call__(self, *args, **kwargs):
+        InstrumentedReplacer.actions.append(('__call__', args, kwargs))
+        return lazy_import.ScopeReplacer.__call__(self, *args, **kwargs)
+
+
+class InstrumentedImportReplacer(lazy_import.ImportReplacer):
+
+    @staticmethod
+    def use_actions(actions):
+        InstrumentedImportReplacer.actions = actions
+
+    def _import(self, scope, name):
+        InstrumentedImportReplacer.actions.append(('_import', name))
+        return lazy_import.ImportReplacer._import(self, scope, name)
+
+    def _replace(self):
+        InstrumentedImportReplacer.actions.append('_replace')
+        return lazy_import.ScopeReplacer._replace(self)
+
+    def __getattribute__(self, attr):
+        InstrumentedImportReplacer.actions.append(('__getattribute__', attr))
+        return lazy_import.ScopeReplacer.__getattribute__(self, attr)
+
+    def __call__(self, *args, **kwargs):
+        InstrumentedImportReplacer.actions.append(('__call__', args, kwargs))
+        return lazy_import.ScopeReplacer.__call__(self, *args, **kwargs)
+
+
+class TestClass(object):
+    """Just a simple test class instrumented for the test cases"""
+
+    class_member = 'class_member'
+
+    @staticmethod
+    def use_actions(actions):
+        TestClass.actions = actions
+
+    def __init__(self):
+        TestClass.actions.append('init')
+
+    def foo(self, x):
+        TestClass.actions.append(('foo', x))
+        return 'foo'
+
+
+class TestScopeReplacer(TestCase):
+    """Test the ability of the replacer to put itself into the correct scope.
+
+    In these tests we use the global scope, because we cannot replace
+    variables in the local scope. This means that we need to be careful
+    and not have the replacing objects use the same name, or we would
+    get collisions.
+    """
+
+    def test_object(self):
+        """ScopeReplacer can create an instance in local scope.
+        
+        An object should appear in globals() by constructing a ScopeReplacer,
+        and it will be replaced with the real object upon the first request.
+        """
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass()
+
+        try:
+            test_obj1
+        except NameError:
+            # test_obj1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj1 was not supposed to exist yet')
+
+        orig_globals = set(globals().keys())
+
+        InstrumentedReplacer(scope=globals(), name='test_obj1',
+                             factory=factory)
+
+        new_globals = set(globals().keys())
+
+        # We can't use isinstance() because that uses test_obj1.__class__
+        # and that goes through __getattribute__ which would activate
+        # the replacement
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj1, '__class__'))
+        self.assertEqual('foo', test_obj1.foo(1))
+        self.assertIsInstance(test_obj1, TestClass)
+        self.assertEqual('foo', test_obj1.foo(2))
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                         ], actions)
+
+    def test_replace_side_effects(self):
+        """Creating a new object should only create one entry in globals.
+
+        And only that entry even after replacement.
+        """
+        try:
+            test_scope1
+        except NameError:
+            # test_scope1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_scope1 was not supposed to exist yet')
+
+        # ignore the logged actions
+        TestClass.use_actions([])
+
+        def factory(replacer, scope, name):
+            return TestClass()
+
+        orig_globals = set(globals().keys())
+
+        lazy_import.ScopeReplacer(scope=globals(), name='test_scope1',
+                                  factory=factory)
+
+        new_globals = set(globals().keys())
+
+        self.assertEqual(lazy_import.ScopeReplacer,
+                         object.__getattribute__(test_scope1, '__class__'))
+        self.assertEqual('foo', test_scope1.foo(1))
+        self.assertIsInstance(test_scope1, TestClass)
+
+        final_globals = set(globals().keys())
+
+        self.assertEqual(set(['test_scope1']), new_globals - orig_globals)
+        self.assertEqual(set(), orig_globals - new_globals)
+        self.assertEqual(set(), final_globals - new_globals)
+        self.assertEqual(set(), new_globals - final_globals)
+
+    def test_class(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass
+
+        try:
+            test_class1
+        except NameError:
+            # test_class2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_class1 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_class1',
+                             factory=factory)
+
+        self.assertEqual('class_member', test_class1.class_member)
+        self.assertEqual(test_class1, TestClass)
+        self.assertEqual([('__getattribute__', 'class_member'),
+                          '_replace',
+                          'factory',
+                         ], actions)
+
+    def test_call_class(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass
+
+        try:
+            test_class2
+        except NameError:
+            # test_class2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_class2 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_class2',
+                             factory=factory)
+
+        self.failIf(test_class2 is TestClass)
+        obj = test_class2()
+        self.assertIs(test_class2, TestClass)
+        self.assertIsInstance(obj, TestClass)
+        self.assertEqual('class_member', obj.class_member)
+        self.assertEqual([('__call__', (), {}),
+                          '_replace',
+                          'factory',
+                          'init',
+                         ], actions)
+
+    def test_call_func(self):
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+
+        def func(a, b, c=None):
+            actions.append('func')
+            return (a, b, c)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return func
+
+        try:
+            test_func1
+        except NameError:
+            # test_func1 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_func1 was not supposed to exist yet')
+        InstrumentedReplacer(scope=globals(), name='test_func1',
+                             factory=factory)
+
+        self.failIf(test_func1 is func)
+        val = test_func1(1, 2, c='3')
+        self.assertIs(test_func1, func)
+
+        self.assertEqual((1,2,'3'), val)
+        self.assertEqual([('__call__', (1,2), {'c':'3'}),
+                          '_replace',
+                          'factory',
+                          'func',
+                         ], actions)
+
+    def test_other_variable(self):
+        """Test when a ScopeReplacer is assigned to another variable.
+
+        This test could be updated if we find a way to trap '=' rather
+        than just giving a belated exception.
+        ScopeReplacer only knows about the variable it was created as,
+        so until the object is replaced, it is illegal to pass it to
+        another variable. (Though discovering this may take a while)
+        """
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass()
+
+        try:
+            test_obj2
+        except NameError:
+            # test_obj2 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj2 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_obj2',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj2, '__class__'))
+        # This is technically not allowed, but we don't have a way to
+        # test it until later.
+        test_obj3 = test_obj2
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj2, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj3, '__class__'))
+        
+        # The first use of the alternate variable causes test_obj2 to
+        # be replaced.
+        self.assertEqual('foo', test_obj3.foo(1))
+        # test_obj2 has been replaced, but the ScopeReplacer has no
+        # idea of test_obj3
+        self.assertEqual(TestClass,
+                         object.__getattribute__(test_obj2, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj3, '__class__'))
+        # We should be able to access test_obj2 attributes normally
+        self.assertEqual('foo', test_obj2.foo(2))
+        self.assertEqual('foo', test_obj2.foo(3))
+
+        # However, the next access on test_obj3 should raise an error
+        # because only now are we able to detect the problem.
+        self.assertRaises(errors.IllegalUseOfScopeReplacer,
+                          getattr, test_obj3, 'foo')
+        
+        # However, the 
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                          ('foo', 3),
+                          ('__getattribute__', 'foo'),
+                          '_replace',
+                         ], actions)
+
+
+class ImportReplacerHelper(TestCaseInTempDir):
+    """Test the ability to have a lazily imported module or object"""
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        self.create_modules()
+        base_path = self.test_dir + '/base'
+
+        self.actions = []
+        InstrumentedImportReplacer.use_actions(self.actions)
+
+        original_import = __import__
+        def instrumented_import(mod, scope1, scope2, fromlist):
+            self.actions.append(('import', mod, fromlist))
+            return original_import(mod, scope1, scope2, fromlist)
+
+        def cleanup():
+            if base_path in sys.path:
+                sys.path.remove(base_path)
+            __builtins__['__import__'] = original_import
+        self.addCleanup(cleanup)
+        sys.path.append(base_path)
+        __builtins__['__import__'] = instrumented_import
+
+    def create_modules(self):
+        """Create some random modules to be imported.
+
+        Each entry has a random suffix, and the full names are saved
+
+        These are setup as follows:
+         base/ <= used to ensure not in default search path
+            root-XXX/
+                __init__.py <= This will contain var1, func1
+                mod-XXX.py <= This will contain var2, func2
+                sub-XXX/
+                    __init__.py <= Contains var3, func3
+                    submoda-XXX.py <= contains var4, func4
+                    submodb-XXX.py <= containse var5, func5
+        """
+        rand_suffix = osutils.rand_chars(4)
+        root_name = 'root_' + rand_suffix
+        mod_name = 'mod_' + rand_suffix
+        sub_name = 'sub_' + rand_suffix
+        submoda_name = 'submoda_' + rand_suffix
+        submodb_name = 'submodb_' + rand_suffix
+
+        os.mkdir('base')
+        root_path = osutils.pathjoin('base', root_name)
+        os.mkdir(root_path)
+        root_init = osutils.pathjoin(root_path, '__init__.py')
+        f = open(osutils.pathjoin(root_path, '__init__.py'), 'wb')
+        try:
+            f.write('var1 = 1\ndef func1(a):\n  return a\n')
+        finally:
+            f.close()
+        mod_path = osutils.pathjoin(root_path, mod_name + '.py')
+        f = open(mod_path, 'wb')
+        try:
+            f.write('var2 = 2\ndef func2(a):\n  return a\n')
+        finally:
+            f.close()
+
+        sub_path = osutils.pathjoin(root_path, sub_name)
+        os.mkdir(sub_path)
+        f = open(osutils.pathjoin(sub_path, '__init__.py'), 'wb')
+        try:
+            f.write('var3 = 3\ndef func3(a):\n  return a\n')
+        finally:
+            f.close()
+        submoda_path = osutils.pathjoin(sub_path, submoda_name + '.py')
+        f = open(submoda_path, 'wb')
+        try:
+            f.write('var4 = 4\ndef func4(a):\n  return a\n')
+        finally:
+            f.close()
+        submodb_path = osutils.pathjoin(sub_path, submodb_name + '.py')
+        f = open(submodb_path, 'wb')
+        try:
+            f.write('var5 = 5\ndef func5(a):\n  return a\n')
+        finally:
+            f.close()
+        self.root_name = root_name
+        self.mod_name = mod_name
+        self.sub_name = sub_name
+        self.submoda_name = submoda_name
+        self.submodb_name = submodb_name
+
+
+class TestImportReplacerHelper(ImportReplacerHelper):
+
+    def test_basic_import(self):
+        """Test that a real import of these modules works"""
+        sub_mod_path = '.'.join([self.root_name, self.sub_name,
+                                  self.submoda_name])
+        root = __import__(sub_mod_path, globals(), locals(), [])
+        self.assertEqual(1, root.var1)
+        self.assertEqual(3, getattr(root, self.sub_name).var3)
+        self.assertEqual(4, getattr(getattr(root, self.sub_name),
+                                    self.submoda_name).var4)
+
+        mod_path = '.'.join([self.root_name, self.mod_name])
+        root = __import__(mod_path, globals(), locals(), [])
+        self.assertEqual(2, getattr(root, self.mod_name).var2)
+
+        self.assertEqual([('import', sub_mod_path, []),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+
+class TestImportReplacer(ImportReplacerHelper):
+
+    def test_import_root(self):
+        """Test 'import root-XXX as root1'"""
+        try:
+            root1
+        except NameError:
+            # root1 shouldn't exist yet
+            pass
+        else:
+            self.fail('root1 was not supposed to exist yet')
+
+        # This should replicate 'import root-xxyyzz as root1'
+        InstrumentedImportReplacer(scope=globals(), name='root1',
+                                   module_path=[self.root_name],
+                                   member=None, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root1, '__class__'))
+        self.assertEqual(1, root1.var1)
+        self.assertEqual('x', root1.func1('x'))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root1'),
+                          ('import', self.root_name, []),
+                         ], self.actions)
+
+    def test_import_mod(self):
+        """Test 'import root-XXX.mod-XXX as mod2'"""
+        try:
+            mod1
+        except NameError:
+            # mod1 shouldn't exist yet
+            pass
+        else:
+            self.fail('mod1 was not supposed to exist yet')
+
+        mod_path = self.root_name + '.' + self.mod_name
+        InstrumentedImportReplacer(scope=globals(), name='mod1',
+                                   module_path=[self.root_name, self.mod_name],
+                                   member=None, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(mod1, '__class__'))
+        self.assertEqual(2, mod1.var2)
+        self.assertEqual('y', mod1.func2('y'))
+
+        self.assertEqual([('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod1'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_mod_from_root(self):
+        """Test 'from root-XXX import mod-XXX as mod2'"""
+        try:
+            mod2
+        except NameError:
+            # mod2 shouldn't exist yet
+            pass
+        else:
+            self.fail('mod2 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(), name='mod2',
+                                   module_path=[self.root_name],
+                                   member=self.mod_name, children={})
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(mod2, '__class__'))
+        self.assertEqual(2, mod2.var2)
+        self.assertEqual('y', mod2.func2('y'))
+
+        self.assertEqual([('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod2'),
+                          ('import', self.root_name, [self.mod_name]),
+                         ], self.actions)
+
+    def test_import_root_and_mod(self):
+        """Test 'import root-XXX.mod-XXX' remapping both to root3.mod3"""
+        try:
+            root3
+        except NameError:
+            # root3 shouldn't exist yet
+            pass
+        else:
+            self.fail('root3 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root3', module_path=[self.root_name], member=None,
+            children={'mod3':([self.root_name, self.mod_name], None, {})})
+
+        # So 'root3' should be a lazy import
+        # and once it is imported, mod3 should also be lazy until
+        # actually accessed.
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root3, '__class__'))
+        self.assertEqual(1, root3.var1)
+
+        # There is a mod3 member, but it is also lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root3.mod3, '__class__'))
+        self.assertEqual(2, root3.mod3.var2)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root3'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod3'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_root_and_root_mod(self):
+        """Test that 'import root, root.mod' can be done.
+
+        The second import should re-use the first one, and just add
+        children to be imported.
+        """
+        try:
+            root4
+        except NameError:
+            # root4 shouldn't exist yet
+            pass
+        else:
+            self.fail('root4 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root4', module_path=[self.root_name], member=None,
+            children={})
+
+        # So 'root4' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root4, '__class__'))
+
+        # Lets add a new child to be imported on demand
+        # This syntax of using object.__getattribute__ is the correct method
+        # for accessing the _import_replacer_children member
+        children = object.__getattribute__(root4, '_import_replacer_children')
+        children['mod4'] = ([self.root_name, self.mod_name], None, {})
+
+        # Accessing root4.mod4 should import root, but mod should stay lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root4.mod4, '__class__'))
+        self.assertEqual(2, root4.mod4.var2)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        self.assertEqual([('__getattribute__', 'mod4'),
+                          '_replace',
+                          ('_import', 'root4'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod4'),
+                          ('import', mod_path, []),
+                         ], self.actions)
+
+    def test_import_root_sub_submod(self):
+        """Test import root.mod, root.sub.submoda, root.sub.submodb
+        root should be a lazy import, with multiple children, who also
+        have children to be imported.
+        And when root is imported, the children should be lazy, and
+        reuse the intermediate lazy object.
+        """
+        try:
+            root5
+        except NameError:
+            # root5 shouldn't exist yet
+            pass
+        else:
+            self.fail('root5 was not supposed to exist yet')
+
+        InstrumentedImportReplacer(scope=globals(),
+            name='root5', module_path=[self.root_name], member=None,
+            children={'mod5':([self.root_name, self.mod_name], None, {}),
+                      'sub5':([self.root_name, self.sub_name], None,
+                            {'submoda5':([self.root_name, self.sub_name,
+                                         self.submoda_name], None, {}),
+                             'submodb5':([self.root_name, self.sub_name,
+                                          self.submodb_name], None, {})
+                            }),
+                     })
+
+        # So 'root5' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5, '__class__'))
+
+        # Accessing root5.mod5 should import root, but mod should stay lazy
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5.mod5, '__class__'))
+        # root5.sub5 should still be lazy, but not re-import root5
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root5.sub5, '__class__'))
+
+        # Accessing root5.sub5.submoda5 should import sub5, but not either
+        # of the sub objects (they should be available as lazy objects
+        self.assertEqual(InstrumentedImportReplacer,
+                     object.__getattribute__(root5.sub5.submoda5, '__class__'))
+        self.assertEqual(InstrumentedImportReplacer,
+                     object.__getattribute__(root5.sub5.submodb5, '__class__'))
+
+        # This should import mod5
+        self.assertEqual(2, root5.mod5.var2)
+        # These should import submoda5 and submodb5
+        self.assertEqual(4, root5.sub5.submoda5.var4)
+        self.assertEqual(5, root5.sub5.submodb5.var5)
+
+        mod_path = self.root_name + '.' + self.mod_name
+        sub_path = self.root_name + '.' + self.sub_name
+        submoda_path = sub_path + '.' + self.submoda_name
+        submodb_path = sub_path + '.' + self.submodb_name
+
+        self.assertEqual([('__getattribute__', 'mod5'),
+                          '_replace',
+                          ('_import', 'root5'),
+                          ('import', self.root_name, []),
+                          ('__getattribute__', 'submoda5'),
+                          '_replace',
+                          ('_import', 'sub5'),
+                          ('import', sub_path, []),
+                          ('__getattribute__', 'var2'),
+                          '_replace',
+                          ('_import', 'mod5'),
+                          ('import', mod_path, []),
+                          ('__getattribute__', 'var4'),
+                          '_replace',
+                          ('_import', 'submoda5'),
+                          ('import', submoda_path, []),
+                          ('__getattribute__', 'var5'),
+                          '_replace',
+                          ('_import', 'submodb5'),
+                          ('import', submodb_path, []),
+                         ], self.actions)
+
+
+class TestConvertImportToMap(TestCase):
+    """Directly test the conversion from import strings to maps"""
+
+    def check(self, expected, import_strings):
+        proc = lazy_import.ImportProcessor()
+        for import_str in import_strings:
+            proc._convert_import_str(import_str)
+        self.assertEqual(expected, proc.imports,
+                         'Import of %r was not converted correctly'
+                         ' %s != %s' % (import_strings, expected,
+                                        proc.imports))
+
+    def test_import_one(self):
+        self.check({'one':(['one'], None, {}),
+                   }, ['import one'])
+
+    def test_import_one_two(self):
+        one_two_map = {'one':(['one'], None,
+                              {'two':(['one', 'two'], None, {}),
+                              }),
+                      }
+        self.check(one_two_map, ['import one.two'])
+        self.check(one_two_map, ['import one, one.two'])
+        self.check(one_two_map, ['import one', 'import one.two'])
+        self.check(one_two_map, ['import one.two', 'import one'])
+
+    def test_import_one_two_three(self):
+        one_two_three_map = {
+            'one':(['one'], None,
+                   {'two':(['one', 'two'], None,
+                           {'three':(['one', 'two', 'three'], None, {}),
+                           }),
+                   }),
+        }
+        self.check(one_two_three_map, ['import one.two.three'])
+        self.check(one_two_three_map, ['import one, one.two.three'])
+        self.check(one_two_three_map, ['import one',
+                                              'import one.two.three'])
+        self.check(one_two_three_map, ['import one.two.three',
+                                              'import one'])
+
+    def test_import_one_as_x(self):
+        self.check({'x':(['one'], None, {}),
+                          }, ['import one as x'])
+
+    def test_import_one_two_as_x(self):
+        self.check({'x':(['one', 'two'], None, {}),
+                   }, ['import one.two as x'])
+
+    def test_import_mixed(self):
+        mixed = {'x':(['one', 'two'], None, {}),
+                 'one':(['one'], None,
+                       {'two':(['one', 'two'], None, {}),
+                       }),
+                }
+        self.check(mixed, ['import one.two as x, one.two'])
+        self.check(mixed, ['import one.two as x', 'import one.two'])
+        self.check(mixed, ['import one.two', 'import one.two as x'])
+
+    def test_import_with_as(self):
+        self.check({'fast':(['fast'], None, {})}, ['import fast'])
+
+
+class TestFromToMap(TestCase):
+    """Directly test the conversion of 'from foo import bar' syntax"""
+
+    def check_result(self, expected, from_strings):
+        proc = lazy_import.ImportProcessor()
+        for from_str in from_strings:
+            proc._convert_from_str(from_str)
+        self.assertEqual(expected, proc.imports,
+                         'Import of %r was not converted correctly'
+                         ' %s != %s' % (from_strings, expected, proc.imports))
+
+    def test_from_one_import_two(self):
+        self.check_result({'two':(['one'], 'two', {})},
+                          ['from one import two'])
+
+    def test_from_one_import_two_as_three(self):
+        self.check_result({'three':(['one'], 'two', {})},
+                          ['from one import two as three'])
+
+    def test_from_one_import_two_three(self):
+        two_three_map = {'two':(['one'], 'two', {}),
+                         'three':(['one'], 'three', {}),
+                        }
+        self.check_result(two_three_map,
+                          ['from one import two, three'])
+        self.check_result(two_three_map,
+                          ['from one import two',
+                           'from one import three'])
+
+    def test_from_one_two_import_three(self):
+        self.check_result({'three':(['one', 'two'], 'three', {})},
+                          ['from one.two import three'])
+
+
+class TestCanonicalize(TestCase):
+    """Test that we can canonicalize import texts"""
+
+    def check(self, expected, text):
+        proc = lazy_import.ImportProcessor()
+        parsed = proc._canonicalize_import_text(text)
+        self.assertEqual(expected, parsed,
+                         'Incorrect parsing of text:\n%s\n%s\n!=\n%s'
+                         % (text, expected, parsed))
+
+    def test_import_one(self):
+        self.check(['import one'], 'import one')
+        self.check(['import one'], '\nimport one\n\n')
+
+    def test_import_one_two(self):
+        self.check(['import one, two'], 'import one, two')
+        self.check(['import one, two'], '\nimport one, two\n\n')
+
+    def test_import_one_as_two_as(self):
+        self.check(['import one as x, two as y'], 'import one as x, two as y')
+        self.check(['import one as x, two as y'],
+                   '\nimport one as x, two as y\n')
+
+    def test_from_one_import_two(self):
+        self.check(['from one import two'], 'from one import two')
+        self.check(['from one import two'], '\nfrom one import two\n\n')
+        self.check(['from one import two'], '\nfrom one import (two)\n')
+        self.check(['from one import  two '], '\nfrom one import (\n\ttwo\n)\n')
+
+    def test_multiple(self):
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\nimport two, three\nfrom one import four')
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\nimport (two, three)\nfrom one import four')
+        self.check(['import one', 'import two, three', 'from one import four'],
+                   'import one\n'
+                   'import two, three\n'
+                   'from one import four')
+        self.check(['import one',
+                    'import two, three', 'from one import  four, '],
+                   'import one\n'
+                   'import two, three\n'
+                   'from one import (\n'
+                   '    four,\n'
+                   '    )\n'
+                   )
+
+    def test_missing_trailing(self):
+        proc = lazy_import.ImportProcessor()
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._canonicalize_import_text,
+                          "from foo import (\n  bar\n")
+
+
+class TestImportProcessor(TestCase):
+    """Test that ImportProcessor can turn import texts into lazy imports"""
+
+    def check(self, expected, text):
+        proc = lazy_import.ImportProcessor()
+        proc._build_map(text)
+        self.assertEqual(expected, proc.imports,
+                         'Incorrect processing of:\n%s\n%s\n!=\n%s'
+                         % (text, expected, proc.imports))
+
+    def test_import_one(self):
+        exp = {'one':(['one'], None, {})}
+        self.check(exp, 'import one')
+        self.check(exp, '\nimport one\n')
+
+    def test_import_one_two(self):
+        exp = {'one':(['one'], None,
+                      {'two':(['one', 'two'], None, {}),
+                      }),
+              }
+        self.check(exp, 'import one.two')
+        self.check(exp, 'import one, one.two')
+        self.check(exp, 'import one\nimport one.two')
+
+    def test_import_as(self):
+        exp = {'two':(['one'], None, {})}
+        self.check(exp, 'import one as two')
+
+    def test_import_many(self):
+        exp = {'one':(['one'], None,
+                      {'two':(['one', 'two'], None,
+                              {'three':(['one', 'two', 'three'], None, {}),
+                              }),
+                       'four':(['one', 'four'], None, {}),
+                      }),
+               'five':(['one', 'five'], None, {}),
+              }
+        self.check(exp, 'import one.two.three, one.four, one.five as five')
+        self.check(exp, 'import one.five as five\n'
+                        'import one\n'
+                        'import one.two.three\n'
+                        'import one.four\n')
+
+    def test_from_one_import_two(self):
+        exp = {'two':(['one'], 'two', {})}
+        self.check(exp, 'from one import two\n')
+        self.check(exp, 'from one import (\n'
+                        '    two,\n'
+                        '    )\n')
+
+    def test_from_one_import_two(self):
+        exp = {'two':(['one'], 'two', {})}
+        self.check(exp, 'from one import two\n')
+        self.check(exp, 'from one import (two)\n')
+        self.check(exp, 'from one import (two,)\n')
+        self.check(exp, 'from one import two as two\n')
+        self.check(exp, 'from one import (\n'
+                        '    two,\n'
+                        '    )\n')
+
+    def test_from_many(self):
+        exp = {'two':(['one'], 'two', {}),
+               'three':(['one', 'two'], 'three', {}),
+               'five':(['one', 'two'], 'four', {}),
+              }
+        self.check(exp, 'from one import two\n'
+                        'from one.two import three, four as five\n')
+        self.check(exp, 'from one import two\n'
+                        'from one.two import (\n'
+                        '    three,\n'
+                        '    four as five,\n'
+                        '    )\n')
+
+    def test_mixed(self):
+        exp = {'two':(['one'], 'two', {}),
+               'three':(['one', 'two'], 'three', {}),
+               'five':(['one', 'two'], 'four', {}),
+               'one':(['one'], None,
+                      {'two':(['one', 'two'], None, {}),
+                      }),
+              }
+        self.check(exp, 'from one import two\n'
+                        'from one.two import three, four as five\n'
+                        'import one.two')
+        self.check(exp, 'from one import two\n'
+                        'from one.two import (\n'
+                        '    three,\n'
+                        '    four as five,\n'
+                        '    )\n'
+                        'import one\n'
+                        'import one.two\n')
+
+    def test_incorrect_line(self):
+        proc = lazy_import.ImportProcessor()
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'foo bar baz')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'improt foo')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'importfoo')
+        self.assertRaises(errors.InvalidImportLine,
+                          proc._build_map, 'fromimport')
+
+    def test_name_collision(self):
+        proc = lazy_import.ImportProcessor()
+        proc._build_map('import foo')
+
+        # All of these would try to create an object with the
+        # same name as an existing object.
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'import bar as foo')
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'from foo import bar as foo')
+        self.assertRaises(errors.ImportNameCollision,
+                          proc._build_map, 'from bar import foo')
+
+
+class TestLazyImportProcessor(ImportReplacerHelper):
+
+    def test_root(self):
+        try:
+            root6
+        except NameError:
+            pass # root6 should not be defined yet
+        else:
+            self.fail('root6 was not supposed to exist yet')
+
+        text = 'import %s as root6' % (self.root_name,)
+        proc = lazy_import.ImportProcessor(InstrumentedImportReplacer)
+        proc.lazy_import(scope=globals(), text=text)
+
+        # So 'root6' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root6, '__class__'))
+
+        self.assertEqual(1, root6.var1)
+        self.assertEqual('x', root6.func1('x'))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root6'),
+                          ('import', self.root_name, []),
+                         ], self.actions)
+
+    def test_import_deep(self):
+        """Test import root.mod, root.sub.submoda, root.sub.submodb
+        root should be a lazy import, with multiple children, who also
+        have children to be imported.
+        And when root is imported, the children should be lazy, and
+        reuse the intermediate lazy object.
+        """
+        try:
+            submoda7
+        except NameError:
+            pass # submoda7 should not be defined yet
+        else:
+            self.fail('submoda7 was not supposed to exist yet')
+
+        text = """\
+import %(root_name)s.%(sub_name)s.%(submoda_name)s as submoda7
+""" % self.__dict__
+        proc = lazy_import.ImportProcessor(InstrumentedImportReplacer)
+        proc.lazy_import(scope=globals(), text=text)
+
+        # So 'submoda7' should be a lazy import
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(submoda7, '__class__'))
+
+        # This should import submoda7
+        self.assertEqual(4, submoda7.var4)
+
+        sub_path = self.root_name + '.' + self.sub_name
+        submoda_path = sub_path + '.' + self.submoda_name
+
+        self.assertEqual([('__getattribute__', 'var4'),
+                          '_replace',
+                          ('_import', 'submoda7'),
+                          ('import', submoda_path, []),
+                         ], self.actions)
+
+    def test_lazy_import(self):
+        """Smoke test that lazy_import() does the right thing"""
+        try:
+            root8
+        except NameError:
+            pass # root8 should not be defined yet
+        else:
+            self.fail('root8 was not supposed to exist yet')
+        lazy_import.lazy_import(globals(),
+                                'import %s as root8' % (self.root_name,),
+                                lazy_import_class=InstrumentedImportReplacer)
+
+        self.assertEqual(InstrumentedImportReplacer,
+                         object.__getattribute__(root8, '__class__'))
+
+        self.assertEqual(1, root8.var1)
+        self.assertEqual(1, root8.var1)
+        self.assertEqual(1, root8.func1(1))
+
+        self.assertEqual([('__getattribute__', 'var1'),
+                          '_replace',
+                          ('_import', 'root8'),
+                          ('import', self.root_name, []),
+                         ], self.actions)

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2006-09-06 02:47:54 +0000
+++ bzrlib/tests/test_merge.py	2006-09-11 09:34:51 +0000
@@ -127,7 +127,7 @@
         transform_tree(tree, tree.branch.basis_tree())
 
     def test_ignore_zero_merge_inner(self):
-        # Test that merge_inner's ignore zero paramter is effective
+        # Test that merge_inner's ignore zero parameter is effective
         tree_a =self.make_branch_and_tree('a')
         tree_a.commit(message="hello")
         dir_b = tree_a.bzrdir.sprout('b')

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-08-30 14:37:31 +0000
+++ bzrlib/tests/test_merge_core.py	2006-09-10 23:42:17 +0000
@@ -25,9 +25,11 @@
            path = pathjoin(self.dir, name)
            os.mkdir(path)
            wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(path)
+           # the tests perform pulls, so need a branch that is writeable.
+           wt.lock_write()
            tt = TreeTransform(wt)
            return wt, tt
-        self.base, self.base_tt = wt('base') 
+        self.base, self.base_tt = wt('base')
         self.this, self.this_tt = wt('this')
         self.other, self.other_tt = wt('other')
 
@@ -163,6 +165,9 @@
         return new_inventory
 
     def cleanup(self):
+        self.base.unlock()
+        self.this.unlock()
+        self.other.unlock()
         rmtree(self.dir)
 
 
@@ -195,6 +200,7 @@
         builder.merge(interesting_ids=["1"])
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
         self.assertEqual(builder.this.get_file("2").read(), "hello1" )
+        builder.cleanup()
         
     def test_file_moves(self):
         """Test moves"""
@@ -257,8 +263,7 @@
 >>>>>>> MERGE-SOURCE
 """
         self.assertEqualDiff(builder.this.get_file("a").read(), expected)
-
- 
+        builder.cleanup()
 
     def do_contents_test(self, merge_factory):
         """Test merging with specified ContentsChange factory"""
@@ -371,6 +376,7 @@
                          base=False)
         conflicts = builder.merge()
         self.assertEqual(conflicts, []) 
+        builder.cleanup()
 
 
 class FunctionalMergeTest(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_permissions.py	2006-09-09 18:52:57 +0000
@@ -34,6 +34,7 @@
 import sys
 import stat
 from cStringIO import StringIO
+import urllib
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
@@ -72,10 +73,11 @@
         test.assertTransportMode(t, base, dir_mode)
     for root, dirs, files in os.walk(base):
         for d in dirs:
-            p = os.path.join(root, d)
+            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [d]])
             test.assertTransportMode(t, p, dir_mode)
         for f in files:
             p = os.path.join(root, f)
+            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [f]])
             test.assertTransportMode(t, p, file_mode)
 
 
@@ -88,7 +90,8 @@
         t = self.make_branch_and_tree('.')
         b = t.branch
         open('a', 'wb').write('foo\n')
-        t.add('a')
+        # ensure check_mode_r works with capital-letter file-ids like TREE_ROOT
+        t.add('a', 'CAPS-ID')
         t.commit('foo')
 
         chmod_r('.bzr', 0644, 0755)

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_repository.py	2006-09-09 18:52:57 +0000
@@ -38,13 +38,15 @@
 from bzrlib.transport import get_transport
 from bzrlib.transport.http import HttpServer
 from bzrlib.transport.memory import MemoryServer
+from bzrlib import upgrade, workingtree
 
 
 class TestDefaultFormat(TestCase):
 
     def test_get_set_default_format(self):
+        private_default = repository._default_format.__class__
         old_format = repository.RepositoryFormat.get_default_format()
-        self.assertTrue(isinstance(old_format, repository.RepositoryFormatKnit1))
+        self.assertTrue(isinstance(old_format, private_default))
         repository.RepositoryFormat.set_default_format(SampleRepositoryFormat())
         # creating a repository should now create an instrumented dir.
         try:
@@ -435,3 +437,25 @@
     def test_unescape_xml(self):
         """We get some kind of error when malformed entities are passed"""
         self.assertRaises(KeyError, repository._unescape_xml, 'foo&bar;') 
+
+
+class TestRepositoryFormatKnit2(TestCaseWithTransport):
+
+    def test_convert(self):
+        """Ensure the upgrade adds weaves for roots"""
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit1()
+        tree = self.make_branch_and_tree('.', format)
+        tree.commit("Dull commit", rev_id="dull")
+        revision_tree = tree.branch.repository.revision_tree('dull')
+        self.assertRaises(errors.NoSuchFile, revision_tree.get_file_lines,
+            revision_tree.inventory.root.file_id)
+        format = bzrdir.BzrDirMetaFormat1()
+        format.repository_format = repository.RepositoryFormatKnit2()
+        upgrade.Convert('.', format)
+        tree = workingtree.WorkingTree.open('.')
+        revision_tree = tree.branch.repository.revision_tree('dull')
+        revision_tree.get_file_lines(revision_tree.inventory.root.file_id)
+        tree.commit("Another dull commit", rev_id='dull2')
+        revision_tree = tree.branch.repository.revision_tree('dull2')
+        self.assertEqual('dull', revision_tree.inventory.root.revision)

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/test_tree.py	2006-08-22 16:14:17 +0000
@@ -79,7 +79,7 @@
         """This test tests the way Tree.compare() uses InterTree."""
         old_optimisers = InterTree._optimisers
         try:
-            InterTree._optimisers = set()
+            InterTree._optimisers = []
             RecordingOptimiser.calls = []
             InterTree.register_optimiser(RecordingOptimiser)
             tree = self.make_branch_and_tree('1')

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-09-05 20:58:28 +0000
+++ bzrlib/tests/test_workingtree.py	2006-09-10 22:50:11 +0000
@@ -26,11 +26,17 @@
 from bzrlib.errors import NotBranchError, NotVersionedError
 from bzrlib.lockdir import LockDir
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter
 from bzrlib.transport import get_transport
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree)
+from bzrlib.workingtree import (
+    needs_tree_write_lock,
+    TreeEntry,
+    TreeDirectory,
+    TreeFile,
+    TreeLink,
+    WorkingTree,
+    )
 
 class TestTreeDirectory(TestCaseWithTransport):
 
@@ -364,3 +370,49 @@
         finally:
             bzrlib.DEFAULT_IGNORE = orig_default
             ignores._runtime_ignores = orig_runtime
+
+
+class InstrumentedTree(object):
+    """A instrumented tree to check the needs_tree_write_lock decorator."""
+
+    def __init__(self):
+        self._locks = []
+
+    def lock_tree_write(self):
+        self._locks.append('t')
+
+    @needs_tree_write_lock
+    def method_with_tree_write_lock(self, *args, **kwargs):
+        """A lock_tree_write decorated method that returns its arguments."""
+        return args, kwargs
+
+    @needs_tree_write_lock
+    def method_that_raises(self):
+        """This method causes an exception when called with parameters.
+        
+        This allows the decorator code to be checked - it should still call
+        unlock.
+        """
+
+    def unlock(self):
+        self._locks.append('u')
+
+
+class TestInstrumentedTree(TestCase):
+
+    def test_needs_tree_write_lock(self):
+        """@needs_tree_write_lock should be semantically transparent."""
+        tree = InstrumentedTree()
+        self.assertEqual(
+            'method_with_tree_write_lock',
+            tree.method_with_tree_write_lock.__name__)
+        self.assertEqual(
+            "A lock_tree_write decorated method that returns its arguments.",
+            tree.method_with_tree_write_lock.__doc__)
+        args = (1, 2, 3)
+        kwargs = {'a':'b'}
+        result = tree.method_with_tree_write_lock(1,2,3, a='b')
+        self.assertEqual((args, kwargs), result)
+        self.assertEqual(['t', 'u'], tree._locks)
+        self.assertRaises(TypeError, tree.method_that_raises, 'foo')
+        self.assertEqual(['t', 'u', 't', 'u'], tree._locks)

=== modified file 'bzrlib/tests/workingtree_implementations/test_basis_inventory.py'
--- bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-08-17 14:16:46 +0000
+++ bzrlib/tests/workingtree_implementations/test_basis_inventory.py	2006-09-09 18:52:57 +0000
@@ -18,8 +18,9 @@
 
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.branch import Branch
+from bzrlib import inventory
 from bzrlib.revision import Revision
-import bzrlib.xml5
+import bzrlib.xml6
 
 
 class TestBasisInventory(TestCaseWithWorkingTree):
@@ -35,7 +36,7 @@
         t.add('a')
         t.commit('a', rev_id='r1')
 
-        t._control_files.get_utf8('basis-inventory')
+        t._control_files.get_utf8('basis-inventory-cache')
 
         basis_inv = t.basis_tree().inventory
         self.assertEquals('r1', basis_inv.revision_id)
@@ -47,16 +48,32 @@
         t.add('b')
         t.commit('b', rev_id='r2')
 
-        t._control_files.get_utf8('basis-inventory')
+        t._control_files.get_utf8('basis-inventory-cache')
 
         basis_inv_txt = t.read_basis_inventory()
-        basis_inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(basis_inv_txt)
-        basis_inv.root.revision = 'r2'
+        basis_inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(basis_inv_txt)
         self.assertEquals('r2', basis_inv.revision_id)
         store_inv = b.repository.get_inventory('r2')
 
         self.assertEquals(store_inv._byid, basis_inv._byid)
 
+    def test_wrong_format(self):
+        """WorkingTree.basis safely ignores junk basis inventories"""
+        t = self.make_branch_and_tree('.')
+        b = t.branch
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+        t._control_files.put_utf8('basis-inventory-cache', 'booga')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', '<xml/>')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', '<inventory />')
+        t.basis_tree()
+        t._control_files.put_utf8('basis-inventory-cache', 
+                                  '<inventory format="pi"/>')
+        t.basis_tree()
+
     def test_basis_inv_gets_revision(self):
         """When the inventory of the basis tree has no revision id it gets set.
 
@@ -65,11 +82,13 @@
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         # TODO change this to use CommitBuilder
+        inv = inventory.Inventory(revision_id='r1')
+        inv.root.revision = 'r1'
+        inv_lines = tree.branch.repository.serialise_inventory(inv).split('\n')
+        inv_lines = [(l + '\n') for l in inv_lines if l is not None]
         tree.branch.repository.control_weaves.get_weave('inventory',
             tree.branch.repository.get_transaction()
-            ).add_lines('r1', [], [
-                '<inventory format="5">\n',
-                '</inventory>\n'])
+            ).add_lines('r1', [], inv_lines)
         rev = Revision(timestamp=0,
                        timezone=None,
                        committer="Foo Bar <foo@example.com>",
@@ -86,6 +105,6 @@
         # without parsing, but to do this properly needs a serialiser on the
         # tree object that abstracts whether it is xml/rio/etc.
         self.assertContainsRe(
-            tree._control_files.get_utf8('basis-inventory').read(),
+            tree._control_files.get_utf8('basis-inventory-cache').read(),
             'revision_id="r1"')
         

=== modified file 'bzrlib/tests/workingtree_implementations/test_locking.py'
--- bzrlib/tests/workingtree_implementations/test_locking.py	2006-07-24 02:07:58 +0000
+++ bzrlib/tests/workingtree_implementations/test_locking.py	2006-09-10 22:50:11 +0000
@@ -53,6 +53,50 @@
         self.assertFalse(wt.is_locked())
         self.assertFalse(wt.branch.is_locked())
         
+    def test_trivial_lock_tree_write_unlock(self):
+        """Locking for tree write is ok when the branch is not locked."""
+        wt = self.make_branch_and_tree('.')
+
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        wt.lock_tree_write()
+        try:
+            self.assertTrue(wt.is_locked())
+            self.assertTrue(wt.branch.is_locked())
+        finally:
+            wt.unlock()
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        
+    def test_trivial_lock_tree_write_branch_read_locked(self):
+        """It is ok to lock_tree_write when the branch is read locked."""
+        wt = self.make_branch_and_tree('.')
+
+        self.assertFalse(wt.is_locked())
+        self.assertFalse(wt.branch.is_locked())
+        wt.branch.lock_read()
+        try:
+            wt.lock_tree_write()
+        except errors.ReadOnlyError:
+            # When ReadOnlyError is raised, it indicates that the 
+            # workingtree shares its lock with the branch, which is what
+            # the git/hg/bzr0.6 formats do.
+            # in this case, no lock should have been taken - but the tree
+            # will have been locked because they share a lock. Unlocking
+            # just the branch should make everything match again correctly.
+            wt.branch.unlock()
+            self.assertFalse(wt.is_locked())
+            self.assertFalse(wt.branch.is_locked())
+            return
+        try:
+            self.assertTrue(wt.is_locked())
+            self.assertTrue(wt.branch.is_locked())
+        finally:
+            wt.unlock()
+        self.assertFalse(wt.is_locked())
+        self.assertTrue(wt.branch.is_locked())
+        wt.branch.unlock()
+        
     def test_unlock_branch_failures(self):
         """If the branch unlock fails the tree must still unlock."""
         # The public interface for WorkingTree requires a branch, but
@@ -143,3 +187,30 @@
                     wt.unlock()
         finally:
             branch_copy.unlock()
+
+    def test_failing_to_lock_tree_write_branch_does_not_lock(self):
+        """If the branch cannot be read locked, dont lock the tree."""
+        # Many implementations treat read-locks as non-blocking, but some
+        # treat them as blocking with writes.. Accordingly we test this by
+        # opening the branch twice, and locking the branch for write in the
+        # second instance.  Our lock contract requires separate instances to
+        # mutually exclude if a lock is exclusive at all: If we get no error
+        # locking, the test still passes.
+        wt = self.make_branch_and_tree('.')
+        branch_copy = branch.Branch.open('.')
+        branch_copy.lock_write()
+        try:
+            try:
+                wt.lock_tree_write()
+            except errors.LockError:
+                # any error here means the locks are exclusive in some 
+                # manner
+                self.assertFalse(wt.is_locked())
+                self.assertFalse(wt.branch.is_locked())
+                return
+            else:
+                # no error - the branch allows read locks while writes
+                # are taken, just pass.
+                wt.unlock()
+        finally:
+            branch_copy.unlock()

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-09-08 18:52:17 +0000
+++ bzrlib/transform.py	2006-09-10 23:42:17 +0000
@@ -73,13 +73,13 @@
      * set_executability
     """
     def __init__(self, tree, pb=DummyProgress()):
-        """Note: a write lock is taken on the tree.
+        """Note: a tree_write lock is taken on the tree.
         
         Use TreeTransform.finalize() to release the lock
         """
         object.__init__(self)
         self._tree = tree
-        self._tree.lock_write()
+        self._tree.lock_tree_write()
         try:
             control_files = self._tree._control_files
             self._limbodir = urlutils.local_path_from_url(

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2006-09-08 18:46:29 +0000
+++ bzrlib/transport/http/_urllib.py	2006-09-12 10:54:46 +0000
@@ -72,9 +72,9 @@
                     e.errno, errno.errorcode.get(e.errno), path)
                 if e.errno == errno.ENOENT:
                     raise NoSuchFile(path, extra=e)
-            raise ConnectionError(msg = "Error retrieving %s: %s" 
-                             % (self.abspath(relpath), str(e)),
-                             orig_error=e)
+            raise ConnectionError(msg = "Error retrieving %s: %s"
+                                  % (self.abspath(relpath), str(e)),
+                                  orig_error=e)
 
     def _get_url_impl(self, url, method, ranges, tail_amount=0):
         """Actually pass get request into urllib
@@ -113,10 +113,13 @@
             f.read()
             f.close()
             return True
+        except urllib2.HTTPError, e:
+            mutter('url error code: %s, for has url: %r', e.code, abspath)
+            if e.code == 404:
+                return False
+            raise
         except urllib2.URLError, e:
-            mutter('url error code: %s for has url: %r', e.code, abspath)
-            if e.code == 404:
-                return False
+            mutter('url error: %s, for has url: %r', e.reason, abspath)
             raise
         except IOError, e:
             mutter('io error: %s %s for has url: %r',

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-09-08 18:46:29 +0000
+++ bzrlib/tree.py	2006-09-09 18:52:57 +0000
@@ -375,7 +375,7 @@
     will pass through to InterTree as appropriate.
     """
 
-    _optimisers = set()
+    _optimisers = []
 
     @needs_read_lock
     def compare(self, want_unchanged=False, specific_files=None,

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2006-03-23 14:51:52 +0000
+++ bzrlib/upgrade.py	2006-08-17 19:51:54 +0000
@@ -19,7 +19,7 @@
 # change upgrade from .bzr to create a '.bzr-new', then do a bait and switch.
 
 
-from bzrlib.bzrdir import ConvertBzrDir4To5, ConvertBzrDir5To6, BzrDir, BzrDirFormat4, BzrDirFormat5
+from bzrlib.bzrdir import ConvertBzrDir4To5, ConvertBzrDir5To6, BzrDir, BzrDirFormat4, BzrDirFormat5, BzrDirFormat
 import bzrlib.errors as errors
 from bzrlib.transport import get_transport
 import bzrlib.ui as ui
@@ -54,6 +54,11 @@
         if not self.bzrdir.can_convert_format():
             raise errors.BzrError("cannot upgrade from branch format %s" %
                            self.bzrdir._format)
+        if self.format is None:
+            target_format = BzrDirFormat.get_default_format()
+        else:
+            target_format = self.format
+        self.bzrdir.check_conversion_target(target_format)
         self.pb.note('starting upgrade of %s', self.transport.base)
         self._backup_control_dir()
         while self.bzrdir.needs_format_conversion(self.format):

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-07-28 16:05:23 +0000
+++ bzrlib/versionedfile.py	2006-08-22 16:14:17 +0000
@@ -558,7 +558,7 @@
     InterVersionedFile.get(other).method_name(parameters).
     """
 
-    _optimisers = set()
+    _optimisers = []
     """The available optimised InterVersionedFile types."""
 
     def join(self, pb=None, msg=None, version_ids=None, ignore_missing=False):

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-09-08 18:52:17 +0000
+++ bzrlib/workingtree.py	2006-09-10 23:56:57 +0000
@@ -159,6 +159,19 @@
     return gen_file_id('TREE_ROOT')
 
 
+def needs_tree_write_lock(unbound):
+    """Decorate unbound to take out and release a tree_write lock."""
+    def tree_write_locked(self, *args, **kwargs):
+        self.lock_tree_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    tree_write_locked.__doc__ = unbound.__doc__
+    tree_write_locked.__name__ = unbound.__name__
+    return tree_write_locked
+
+
 class TreeEntry(object):
     """An entry that implements the minimum interface used by commands.
 
@@ -410,13 +423,12 @@
         else:
             try:
                 xml = self.read_basis_inventory()
-                inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
-                inv.root.revision = revision_id
-            except NoSuchFile:
-                inv = None
-            if inv is not None and inv.revision_id == revision_id:
-                return bzrlib.tree.RevisionTree(self.branch.repository, inv,
-                                                revision_id)
+                inv = bzrlib.xml6.serializer_v6.read_inventory_from_string(xml)
+                if inv is not None and inv.revision_id == revision_id:
+                    return bzrlib.tree.RevisionTree(self.branch.repository, 
+                                                    inv, revision_id)
+            except (NoSuchFile, errors.BadInventoryFormat):
+                pass
         # No cached copy available, retrieve from the repository.
         # FIXME? RBC 20060403 should we cache the inventory locally
         # at this point ?
@@ -610,7 +622,7 @@
             mode = os.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add(self, files, ids=None):
         """Make files versioned.
 
@@ -671,7 +683,7 @@
 
         self._write_inventory(inv)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_parent_tree_id(self, revision_id, allow_leftmost_as_ghost=False):
         """Add revision_id as a parent.
 
@@ -687,7 +699,7 @@
         self.set_parent_ids(parents,
             allow_leftmost_as_ghost=len(parents) > 1 or allow_leftmost_as_ghost)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_parent_tree(self, parent_tuple, allow_leftmost_as_ghost=False):
         """Add revision_id, tree tuple as a parent.
 
@@ -709,7 +721,7 @@
         self.set_parent_ids(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_pending_merge(self, *revision_ids):
         # TODO: Perhaps should check at this point that the
         # history of the revision is actually present?
@@ -736,7 +748,7 @@
         """
         return self.get_parent_ids()[1:]
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """Set the parent ids to revision_ids.
         
@@ -760,7 +772,7 @@
         merges = revision_ids[1:]
         self._control_files.put_utf8('pending-merges', '\n'.join(merges))
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         """Set the parents of the working tree.
 
@@ -773,26 +785,26 @@
         self.set_parent_ids([rev for (rev, tree) in parents_list],
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_pending_merges(self, rev_list):
         parents = self.get_parent_ids()
         leftmost = parents[:1]
         new_parents = leftmost + rev_list
         self.set_parent_ids(new_parents)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_merge_modified(self, modified_hashes):
         def iter_stanzas():
             for file_id, hash in modified_hashes.iteritems():
                 yield Stanza(file_id=file_id, hash=hash)
         self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def _put_rio(self, filename, stanzas, header):
         my_file = rio_file(stanzas, header)
         self._control_files.put(filename, my_file)
 
-    @needs_write_lock
+    @needs_write_lock # because merge pulls data into the branch.
     def merge_from_branch(self, branch, to_revision=None):
         """Merge from a branch into this working tree.
 
@@ -976,8 +988,7 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
-
-    @needs_write_lock
+    @needs_tree_write_lock
     def move(self, from_paths, to_name):
         """Rename files.
 
@@ -1046,7 +1057,7 @@
         self._write_inventory(inv)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def rename_one(self, from_rel, to_rel):
         """Rename one file.
 
@@ -1105,7 +1116,7 @@
             if not self.is_ignored(subp):
                 yield subp
     
-    @needs_write_lock
+    @needs_tree_write_lock
     def unversion(self, file_ids):
         """Remove the file ids in file_ids from the current versioned set.
 
@@ -1387,6 +1398,22 @@
             self.branch.unlock()
             raise
 
+    def lock_tree_write(self):
+        """Lock the working tree for write, and the branch for read.
+
+        This is useful for operations which only need to mutate the working
+        tree. Taking out branch write locks is a relatively expensive process
+        and may fail if the branch is on read only media. So branch write locks
+        should only be taken out when we are modifying branch data - such as in
+        operations like commit, pull, uncommit and update.
+        """
+        self.branch.lock_read()
+        try:
+            return self._control_files.lock_write()
+        except:
+            self.branch.unlock()
+            raise
+
     def lock_write(self):
         """See Branch.lock_write, and WorkingTree.unlock."""
         self.branch.lock_write()
@@ -1400,9 +1427,9 @@
         return self._control_files.get_physical_lock_status()
 
     def _basis_inventory_name(self):
-        return 'basis-inventory'
+        return 'basis-inventory-cache'
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_last_revision(self, new_revision):
         """Change the last revision in the working tree."""
         if self._change_last_revision(new_revision):
@@ -1441,11 +1468,13 @@
             # root node id can legitimately look like 'revision_id' but cannot
             # contain a '"'.
             xml = self.branch.repository.get_inventory_xml(new_revision)
-            if not 'revision_id="' in xml.split('\n', 1)[0]:
+            firstline = xml.split('\n', 1)[0]
+            if (not 'revision_id="' in firstline or 
+                'format="6"' not in firstline):
                 inv = self.branch.repository.deserialise_inventory(
                     new_revision, xml)
                 inv.revision_id = new_revision
-                xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
+                xml = bzrlib.xml6.serializer_v6.write_inventory_to_string(inv)
             assert isinstance(xml, str), 'serialised xml must be bytestring.'
             path = self._basis_inventory_name()
             sio = StringIO(xml)
@@ -1468,7 +1497,7 @@
         self._set_inventory(result)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def remove(self, files, verbose=False, to_file=None):
         """Remove nominated files from the working inventory..
 
@@ -1508,7 +1537,7 @@
 
         self._write_inventory(inv)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def revert(self, filenames, old_tree=None, backups=True, 
                pb=DummyProgress()):
         from transform import revert
@@ -1525,7 +1554,7 @@
 
     # XXX: This method should be deprecated in favour of taking in a proper
     # new Inventory object.
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_inventory(self, new_inventory_list):
         from bzrlib.inventory import (Inventory,
                                       InventoryDirectory,
@@ -1548,7 +1577,7 @@
                 raise BzrError("unknown kind %r" % kind)
         self._write_inventory(inv)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_root_id(self, file_id):
         """Set the root id for this tree."""
         inv = self.read_working_inventory()
@@ -1660,7 +1689,7 @@
                                   this_tree=self)
         return result
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def _write_inventory(self, inv):
         """Write inventory as the current inventory."""
         sio = StringIO()
@@ -1710,6 +1739,19 @@
      - uses the branch last-revision.
     """
 
+    def lock_tree_write(self):
+        """See WorkingTree.lock_tree_write().
+
+        In Format2 WorkingTrees we have a single lock for the branch and tree
+        so lock_tree_write() degrades to lock_write().
+        """
+        self.branch.lock_write()
+        try:
+            return self._control_files.lock_write()
+        except:
+            self.branch.unlock()
+            raise
+
     def unlock(self):
         # we share control files:
         if self._hashcache.needs_write and self._control_files._lock_count==3:
@@ -1751,12 +1793,12 @@
             self._control_files.put_utf8('last-revision', revision_id)
             return True
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def set_conflicts(self, conflicts):
         self._put_rio('conflicts', conflicts.to_stanzas(), 
                       CONFLICT_HEADER_1)
 
-    @needs_write_lock
+    @needs_tree_write_lock
     def add_conflicts(self, new_conflicts):
         conflict_set = set(self.conflicts())
         conflict_set.update(set(list(new_conflicts)))
@@ -2010,7 +2052,7 @@
                          _format=self,
                          _bzrdir=a_bzrdir,
                          _control_files=control_files)
-        wt.lock_write()
+        wt.lock_tree_write()
         try:
             wt._write_inventory(inv)
             wt.set_root_id(inv.root.file_id)

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2006-09-08 18:46:29 +0000
+++ bzrlib/xml5.py	2006-09-10 02:18:31 +0000
@@ -97,6 +97,10 @@
     
     __slots__ = []
 
+    support_altered_by_hack = True
+    # This format supports the altered-by hack that reads file ids directly out
+    # of the versionedfile, without doing XML parsing.
+
     def write_inventory_to_string(self, inv):
         """Just call write_inventory with a StringIO and return the value"""
         sio = cStringIO.StringIO()
@@ -150,7 +154,7 @@
         append(_encode_and_escape(ie.file_id))
         append(' name="')
         append(_encode_and_escape(ie.name))
-        if ie.parent_id != ROOT_ID:
+        if self._parent_condition(ie):
             assert isinstance(ie.parent_id, basestring)
             append(' parent_id="')
             append(_encode_and_escape(ie.parent_id))
@@ -169,6 +173,9 @@
         append(" />\n")
         return
 
+    def _parent_condition(self, ie):
+        return ie.parent_id != ROOT_ID
+
     def _pack_revision(self, rev):
         """Revision object -> xml tree"""
         root = Element('revision',
@@ -228,7 +235,7 @@
             inv.add(ie)
         return inv
 
-    def _unpack_entry(self, elt):
+    def _unpack_entry(self, elt, none_parents=False):
         kind = elt.tag
         if not InventoryEntry.versionable_kind(kind):
             raise AssertionError('unsupported entry kind %s' % kind)
@@ -236,7 +243,7 @@
         get_cached = cache_utf8.get_cached_unicode
 
         parent_id = elt.get('parent_id')
-        if parent_id is None:
+        if parent_id is None and not none_parents:
             parent_id = ROOT_ID
         # TODO: jam 20060817 At present, caching file ids costs us too 
         #       much time. It slows down overall read performances from

=== added file 'bzrlib/xml6.py'
--- bzrlib/xml6.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml6.py	2006-08-24 03:35:48 +0000
@@ -0,0 +1,37 @@
+from bzrlib import cache_utf8, inventory, errors, xml5
+
+
+class Serializer_v6(xml5.Serializer_v5):
+
+    def _append_inventory_root(self, append, inv):
+        """Append the inventory root to output."""
+        append('<inventory')
+        append(' format="6"')
+        if inv.revision_id is not None:
+            append(' revision_id="')
+            append(xml5._encode_and_escape(inv.revision_id))
+        append('>\n')
+        self._append_entry(append, inv.root)
+
+    def _parent_condition(self, ie):
+        return ie.parent_id is not None
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element"""
+        if elt.tag != 'inventory':
+            raise errors.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
+        format = elt.get('format')
+        if format != '6':
+            raise errors.UnexpectedInventoryFormat('Invalid format version %r'
+                                                   % format)
+        revision_id = elt.get('revision_id')
+        if revision_id is not None:
+            revision_id = cache_utf8.get_cached_unicode(revision_id)
+        inv = inventory.Inventory(root_id=None, revision_id=revision_id)
+        for e in elt:
+            ie = self._unpack_entry(e, none_parents=True)
+            inv.add(ie)
+        return inv
+
+
+serializer_v6 = Serializer_v6()

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2006-07-28 16:05:23 +0000
+++ bzrlib/xml_serializer.py	2006-09-10 02:18:31 +0000
@@ -36,7 +36,7 @@
                                               fromstring, tostring)
     import util.elementtree as elementtree
 
-from bzrlib.errors import BzrError
+from bzrlib import errors
 
 
 class Serializer(object):
@@ -50,10 +50,16 @@
         return tostring(self._pack_inventory(inv)) + '\n'
 
     def read_inventory_from_string(self, xml_string):
-        return self._unpack_inventory(fromstring(xml_string))
+        try:
+            return self._unpack_inventory(fromstring(xml_string))
+        except SyntaxError, e:
+            raise errors.UnexpectedInventoryFormat(e)
 
     def read_inventory(self, f):
-        return self._unpack_inventory(self._read_element(f))
+        try:
+            return self._unpack_inventory(self._read_element(f))
+        except SyntaxError, e:
+            raise errors.UnexpectedInventoryFormat(e)
 
     def write_revision(self, rev, f):
         self._write_element(self._pack_revision(rev), f)

=== modified file 'profile_imports.py'
--- profile_imports.py	2006-06-19 14:40:19 +0000
+++ profile_imports.py	2006-09-12 22:02:02 +0000
@@ -115,12 +115,15 @@
     frame = sys._getframe(1)
     frame_name = frame.f_globals.get('__name__', '<unknown>')
     extra = ''
-    cur_frame = 1
     if frame_name.endswith('demandload'):
         # If this was demandloaded, we have 3 frames to ignore
         extra = ' (demandload)'
         frame = sys._getframe(4)
-        cur_frame = 4
+        frame_name = frame.f_globals.get('__name__', '<unknown>')
+    elif frame_name.endswith('lazy_import'):
+        # If this was lazily imported, we have 3 frames to ignore
+        extra = ' (lazy)'
+        frame = sys._getframe(4)
         frame_name = frame.f_globals.get('__name__', '<unknown>')
     frame_lineno = frame.f_lineno
 

